/********************************************************************************

PROJECT:		CS 298 Project

FILE:			ControlView.cpp

PURPOSE:		Implements event behaviors for GUI controls.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

********************************************************************************/

#include "stdafx.h"
#include "PCA_Reconstructor.h"
#include "ControlView.h"
#include "PCA_ReconstructorDoc.h"
#include "InputView.h"
#include "OutputView.h"
#include "trainingview.h"
#include "eigenutil.h"
#include "reconstruct.h"
#include "winutil.h"
#include "constants.h"

extern "C"
{
#include "detect.h"
#include ".\controlview.h"
}

IMPLEMENT_DYNCREATE(CControlView, CFormView)

// Static handle to CControlView handle
HWND CControlView::wHnd = 0;

// Default constructor
CControlView::CControlView() : CFormView(CControlView::IDD), 
							   stopFlag(0), isExtracted(0), 
							   isTrained(0), isRunning(0), 
							   isSelected(0), noExtraction(0)
{
}

// Destructor
CControlView::~CControlView()
{
	delete pDlg;
	delete edgeDetectDialog;
	delete filterDialog;
	delete pcaTruncationDialog;
}

// Windows controls initialization routine
void CControlView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TRAINING_LIST, trainingList);
	DDX_Control(pDX, IDC_GROUP1, trainingGroup);
	DDX_Control(pDX, IDC_GROUP2, reconstructGroup);
	DDX_Control(pDX, IDC_LOAD_TRAINING_BUTTON, loadButton);
	DDX_Control(pDX, IDC_TRAIN_BUTTON, trainButton);
	DDX_Control(pDX, IDC_INPUT_IMAGE_BUTTON, inputButton);
	DDX_Control(pDX, IDC_RECONSTRUCT_BUTTON, reconstructButton);
	DDX_Control(pDX, IDC_ANGLE_EDIT, trainingEdit);
	DDX_Control(pDX, IDC_INPUT_ANGLE_EDIT, inputEdit);
	DDX_Control(pDX, IDC_ANGLE_SPIN, trainingSpinCtrl);
	DDX_Control(pDX, IDC_INPUT_ANGLE_SPIN, inputSpinCtrl);
	DDX_Control(pDX, IDC_TRAINING_STATIC, trainingStatic);
	DDX_Control(pDX, IDC_INPUT_STATIC, inputStatic);
	DDX_Control(pDX, IDC_TRAIN_EDGE_CHECK, trainCheckBox);
	DDX_Control(pDX, IDC_INPUT_EDGE_CHECK, inputCheckBox);
	DDX_Control(pDX, IDC_ONLY_FEATURES_CHECK, featureCheckBox);
}

/* Invoked as a initialization routine to initialize various
   windows and data structures. */
void CControlView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();

	// Initializes neural net data path
	//Track_FindAllFaces(0, 0, 0, 0);

	// Allocate progress dialog
	pDlg = new CProgressDialog(this);

	// Allocate dialog to accept edge detection parameters
	edgeDetectDialog = new CParamDialog(this, EDGE_SIGMA, 
		EDGE_THRESHOLD, PARAM_EDGE_DIALOG_TEXT1, 
		PARAM_EDGE_DIALOG_TEXT2, EDGE_DETECT_DIALOG_TITLE, TRUE);

	// Allocate dialog to accept low pass filter parameters
	filterDialog = new CParamDialog(this, DEFAULT_CUTOFF, 
		FILTER_SIZE, PARAM_FILTER_DIALOG_TEXT1, 
		PARAM_FILTER_DIALOG_TEXT2, FILTER_DIALOG_TITLE);

	pcaTruncationDialog = new CPCATruncationDialog(this);

	// Stores static handle to CControlView
	wHnd = m_hWnd;

	// Create dialog boxes
	pDlg->Create(IDD_PROGRESS_DIALOG, this);
	edgeDetectDialog->Create(IDD_PARAM_DIALOG, this);
	filterDialog->Create(IDD_PARAM_DIALOG, this);
	pcaTruncationDialog->Create(IDD_PRINCIPAL_COMPONENTS_DIALOG, this);

	// Initialize training image listbox
	trainingList.InsertColumn(0, TRAINING_LIST_MSG);
	trainingList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);

	// Position controls for proper placement
	positionControls();

	// Set scrolling
	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = DEFAULT_SCROLL;
	SetScrollSizes(MM_TEXT, sizeTotal);

	// Disable appropriate controls
	GetDlgItem(IDC_TRAIN_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_ANGLE_SPIN)->EnableWindow(FALSE);
	GetDlgItem(IDC_RECONSTRUCT_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_INPUT_ANGLE_SPIN)->EnableWindow(FALSE);
	GetDlgItem(IDC_TRAIN_EDGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_INPUT_EDGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_ONLY_FEATURES_CHECK)->EnableWindow(FALSE);
	
}

// GUI control mappings
BEGIN_MESSAGE_MAP(CControlView, CFormView)
	ON_WM_CREATE()
	ON_BN_CLICKED(IDC_LOAD_TRAINING_BUTTON, OnBnClickedLoadTrainingButton)
	ON_BN_CLICKED(IDC_INPUT_IMAGE_BUTTON, OnBnClickedInputImageButton)
	ON_BN_CLICKED(IDC_RECONSTRUCT_BUTTON, OnBnClickedReconstructButton)
	ON_BN_CLICKED(ID_EDGE_PARAM_BUTTON, OnBnClickedEdgeParamButton)
	ON_NOTIFY(NM_DBLCLK, IDC_TRAINING_LIST, OnNMDblclkTrainingList)
	ON_BN_CLICKED(IDC_TRAIN_BUTTON, OnBnClickedTrainButton)
	ON_NOTIFY(UDN_DELTAPOS, IDC_ANGLE_SPIN, OnDeltaposAngleSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_INPUT_ANGLE_SPIN, OnDeltaposInputAngleSpin)
	ON_COMMAND(ID_EDGE_DETECT_PARAMETERS, OnEdgeDetectParameters)
	ON_BN_CLICKED(IDC_TRAIN_EDGE_CHECK, OnBnClickedTrainEdgeCheck)
	ON_BN_CLICKED(IDC_INPUT_EDGE_CHECK, OnBnClickedInputEdgeCheck)
	ON_COMMAND(ID_FILTER_PARAMETERS, OnFilterParameters)
	ON_BN_CLICKED(ID_FILTER_PARAM_BUTTON, OnBnClickedFilterParamButton)
	ON_COMMAND(ID_PCA_TRUNCATION_CONTEXT, OnParametersPCAtruncation)
	ON_BN_CLICKED(ID_PCA_TRUNCATION_BUTTON, OnBnClickedPCATruncationButton)
	ON_COMMAND(ID_IMAGE_RECONSTRUCT, OnImageReconstruct)
	ON_COMMAND(ID_IMAGE_BATCHRECONSTRUCTANDSAVE, OnImageBatchreconstructandsave)
	ON_BN_CLICKED(IDC_ONLY_FEATURES_CHECK, OnBnClickedOnlyFeaturesCheck)
	ON_UPDATE_COMMAND_UI(ID_DISABLE_FEATURE_EXTRACTION, OnUpdateDisableFeatureExtraction)
	ON_COMMAND(ID_DISABLE_FEATURE_EXTRACTION, OnDisableFeatureExtraction)
END_MESSAGE_MAP()


// CControlView diagnostics

#ifdef _DEBUG
void CControlView::AssertValid() const
{
	CFormView::AssertValid();
}

void CControlView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG


// Position GUI controls for various screen resolutions
void CControlView::positionControls()
{
	int width = GetSystemMetrics(SM_CXSCREEN);
	int height = GetSystemMetrics(SM_CYSCREEN);
	int base = width / 4;
	int marginX = width / 100;
	int marginY = height / 50;
	int tGroupX = 0;
	int tGroupY = marginY;
	int tGroupWidth = base;
	int tGroupHeight = (int)((double)height / 1.5);
	int tListX = tGroupX + marginX;
	int tListY = tGroupY + marginY;
	int tListWidth = tGroupWidth - 2 * (tListX - tGroupX);
	int tListHeight = tGroupHeight - marginY - tGroupWidth / 3;
	int buttonWidth = tListWidth / 5;
	int buttonHeight = buttonWidth / 3;
	int tSpacing = (tGroupHeight - marginY - tListHeight) / 6;
	int loadButtonX = tListX;
	int loadButtonY = tListY + tListHeight + tSpacing;
	int tButtonX = tListX + (tListWidth - buttonWidth) / 2;
	int tButtonY = tListY + tListHeight + tSpacing;
	int tEditX = tListX;
	int tEditY = tListY + tListHeight + tSpacing * 4;
	int tEditWidth = tListWidth / 3;
	int tEditHeight = buttonHeight;
	int cbSpacing = tGroupWidth / 5;
	int tCheckBoxX = tGroupX + cbSpacing * 3;
	int tCheckBoxY = tEditY;
	int tCheckBoxWidth = tGroupWidth - cbSpacing * 3;
	int tCheckBoxHeight = tEditHeight;
	int tSpinCtrlX = tEditX + tEditWidth;
	int tSpinCtrlY = tEditY;
	int tSpinCtrlWidth = tEditWidth / 3;
	int tSpinCtrlHeight = tEditHeight;
	int tStaticWidth = tListWidth;
	int tStaticHeight = marginY;
	int tStaticX = tEditX;
	int tStaticY = tEditY - tStaticHeight;
	int rGroupX = tGroupX;
	int rGroupY = tGroupY + tGroupHeight + marginY;
	int rGroupWidth = tGroupWidth;
	int rGroupHeight = height / 8;
	int rButtonWidth = buttonWidth * 2;
	int rSpacing = rGroupHeight / 8;
	int iButtonX = loadButtonX;
	int iButtonY = rGroupY + rSpacing * 2;
	int rButtonX = tButtonX;
	int rButtonY = iButtonY;
	int iEditX = iButtonX;
	int iEditY = rGroupY + rSpacing * 6;
	int iEditWidth = tEditWidth;
	int iEditHeight = tEditHeight;
	int iCheckBoxX = tCheckBoxX;
	int iCheckBoxY = iEditY - marginY;
	int iCheckBoxWidth = tCheckBoxWidth;
	int iCheckBoxHeight = tCheckBoxHeight;
	int iSpinCtrlX = iEditX + iEditWidth;
	int iSpinCtrlY = iEditY;
	int iSpinCtrlWidth = tSpinCtrlWidth;
	int iSpinCtrlHeight = tSpinCtrlHeight;
	int iStaticWidth = tStaticWidth;
	int iStaticHeight = marginY;
	int iStaticX = iEditX;
	int iStaticY = iEditY - iStaticHeight;
	int featureCheckBoxX = iCheckBoxX;
	int featureCheckBoxY = iEditY;
	int featureCheckBoxWidth = tCheckBoxWidth;
	int featureCheckBoxHeight = tCheckBoxHeight;
	trainingGroup.SetWindowPos(0, tGroupX, tGroupY, 
		tGroupWidth, tGroupHeight, SWP_SHOWWINDOW);
	trainingList.SetWindowPos(0, tListX, tListY, 
		tListWidth, tListHeight, SWP_SHOWWINDOW);
	loadButton.SetWindowPos(0, loadButtonX, loadButtonY,
		buttonWidth, buttonHeight, SWP_SHOWWINDOW);
	trainButton.SetWindowPos(0, tButtonX, tButtonY, 
		buttonWidth, buttonHeight, SWP_SHOWWINDOW);
	trainingEdit.SetWindowPos(0, tEditX, tEditY,
		tEditWidth, tEditHeight, SWP_SHOWWINDOW);
	trainCheckBox.SetWindowPos(0, tCheckBoxX, tCheckBoxY,
		tCheckBoxWidth, tCheckBoxHeight, SWP_SHOWWINDOW);
	trainingSpinCtrl.SetWindowPos(0, tSpinCtrlX, tSpinCtrlY,
		tSpinCtrlWidth, tSpinCtrlHeight, SWP_SHOWWINDOW);
	trainingStatic.SetWindowPos(0, tStaticX, tStaticY, 
		tStaticWidth, tStaticHeight, SWP_SHOWWINDOW);
	reconstructGroup.SetWindowPos(0, rGroupX, rGroupY, 
		rGroupWidth, rGroupHeight, SWP_SHOWWINDOW);
	inputButton.SetWindowPos(0, iButtonX, iButtonY,
		buttonWidth, buttonHeight, SWP_SHOWWINDOW);
	reconstructButton.SetWindowPos(0, rButtonX, rButtonY, 
		rButtonWidth, buttonHeight, SWP_SHOWWINDOW);
	inputEdit.SetWindowPos(0, iEditX, iEditY, 
		iEditWidth, iEditHeight, SWP_SHOWWINDOW);
	inputCheckBox.SetWindowPos(0, iCheckBoxX, iCheckBoxY,
		iCheckBoxWidth, iCheckBoxHeight, SWP_SHOWWINDOW);
	inputSpinCtrl.SetWindowPos(0, iSpinCtrlX, iSpinCtrlY,
		iSpinCtrlWidth, iSpinCtrlHeight, SWP_SHOWWINDOW);
	inputStatic.SetWindowPos(0, iStaticX, iStaticY,
		iStaticWidth, iStaticHeight, SWP_SHOWWINDOW);
	featureCheckBox.SetWindowPos(0, featureCheckBoxX,
		featureCheckBoxY, featureCheckBoxWidth, 
		featureCheckBoxHeight, SWP_SHOWWINDOW);
}

// Reset angle of training image to calculated value
void CControlView::resetTrainingAngle()
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();

	/* Get in-plane rotation angle of displayed training image by
	   using the full path name as a key to look up data structure
	   in a map. */
	double angle = doc->imageMap.getAngle(doc->dImage);

	// Display angle
	CString text;
	text.Format("%.2f", angle * RAD_TO_DEG_CONV);
	trainingEdit.SetWindowText(text);
}

// Reset angle display to 0.00
void CControlView::resetInputAngle()
{
	inputEdit.SetWindowText(ZERO_TEXT);
}

// Clears out all data and memory associated with loaded training images
void CControlView::resetTrainingImages()
{	
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	doc->imageMap.clearAll();
	isExtracted = FALSE;
	isTrained = FALSE;
	trainingList.DeleteAllItems();
	trainingList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	doc->tParam.reset();
	((CTrainingView*)doc->getView(RUNTIME_CLASS(CTrainingView)))->clear();
}

// Clears out all data associated with input image
void CControlView::clearInputParameters()
{
	((CPCA_ReconstructorDoc*)GetDocument())->iParam.reset();
	SetDlgItemText(IDC_INPUT_ANGLE_EDIT, ZERO_TEXT);
}

// Used to set status of progress bar in progress dialog box
void setProgress(int progress, const char* text)
{
	::SendDlgItemMessage(CProgressDialog::wHnd, IDC_PROG_BAR, PBM_SETPOS, progress, 0);
	::SetDlgItemText(CProgressDialog::wHnd, IDC_CURRENT_IMAGE_STATIC, text);
}

// CControlView message handlers

/* Opens up directory listing to allow user to select directory
   with training images to be loaded. */
void CControlView::OnBnClickedLoadTrainingButton()
{
	char path[BUFFER];
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();

	/* Open up directory list box to select directory where training
	   images are stored. */
	BOOL flag = CWinUtil::DemoBrowseForFolder(path, DIRECTORY_DIALOG_MSG);
	if (!flag)
		return;
	CFileFind finder;
	CString filePath(path);

	// Display error dialog if file path is not valid
	if (!finder.FindFile(filePath))
	{	MessageBox(INVALID_PATH_MSG, INVALID_PATH_CAPTION);
		return;
	}

	int count = 0;

	// Reset training images and all data associated with training images
	resetTrainingImages();
	CStringArray DestDir1;

	/* Given directory path, parse through all files in directory
	   and search for all image files. */
	CWinUtil::BuildDirList(DestDir1, path);
	for (int i = 0; i < DestDir1.GetSize(); i++)
	{	CStringArray DestDir2;
		CWinUtil::BuildFileList(DestDir2, DestDir1[i]);
		BOOL unagedImageFound = FALSE;
		BOOL agedImageFound = FALSE;
		CString unagedTmp;
		CString agedTmp;
		for (int j = 0; j < DestDir2.GetSize(); j++)
		{	CString str = DestDir2[j];
			int index = str.ReverseFind('.');
			if (index < 2)
				continue;
			if (str[index - 2] == ORIGINAL_IMAGE)
			{	unagedImageFound = TRUE;
				unagedTmp = str;
			}
			else if (str[index - 2] == AGED_IMAGE)
			{	agedImageFound = TRUE;
				agedTmp = str;
			}
		}

		// Display all image paths in training listbox
		if (unagedImageFound && agedImageFound)
		{	doc->imageMap.addDataMapping(DestDir1[i], 
				CWinUtil::GetSubPath(unagedTmp), 
				CWinUtil::GetSubPath(agedTmp));
			trainingList.InsertItem(i, unagedTmp);
			trainingList.InsertItem(2 * i + 1, agedTmp);
			count += 2;
		}
	}
	trainingList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);

	// Enable appropriate controls
	GetDlgItem(IDC_TRAIN_BUTTON)->EnableWindow();
	GetDlgItem(IDC_TRAIN_EDGE_CHECK)->EnableWindow(FALSE);
	GetDlgItem(IDC_ANGLE_SPIN)->EnableWindow(FALSE);

	CString str;
	str.Format("%d training images loaded.", count);
	MessageBox(str, "Training images loaded ...");
}

// Load input image to be age reconstructed
void CControlView::OnBnClickedInputImageButton()
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();

	CFileDialog dialog(1, 0, 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, FILTER);

	// Open dialog box for user to select input image
	if (dialog.DoModal() == IDCANCEL)
		return;

	doc->iImage = dialog.GetPathName();
	clearInputParameters();

	isSelected = FALSE;

	// Get pointer to input view
	CView* view = doc->getView(RUNTIME_CLASS(CInputView));
	((CInputView*)view)->setBorderText(INPUT_IMAGE_MSG, doc->iImage);

	// Display selected input image in input view
	((CInputView*)view)->display(doc->iImage);

	// Clear output view
	view = doc->getView(RUNTIME_CLASS(COutputView));
	((COutputView*)view)->clear();

	// Enable appropriate controls
	GetDlgItem(IDC_RECONSTRUCT_BUTTON)->EnableWindow();
	GetDlgItem(IDC_INPUT_ANGLE_SPIN)->EnableWindow();
	GetDlgItem(IDC_INPUT_EDGE_CHECK)->EnableWindow();
	GetDlgItem(IDC_ONLY_FEATURES_CHECK)->EnableWindow(FALSE);
}

// Stops progress of feature extraction and/or image classifier training
void CControlView::setStopFlag(int flag)
{
	stopFlag = flag;
}

void CControlView::setTrainedFlag(int flag)
{
	isTrained = flag;
}

void CControlView::setExtractedFlag(int flag)
{
	isExtracted = flag;
}

void CControlView::setSelected(int flag)
{
	isSelected = flag;
}

// Indicates when progress had been stopped during extraction/training
BOOL CControlView::displayStopStatus()
{
	if (stopFlag)
	{	AfxMessageBox(PROG_STOP);
		pDlg->ShowWindow(SW_HIDE);
	}
	return stopFlag;
}

// Thread for image reconstruction
UINT reconstructThread(LPVOID lParam)
{
	CControlView* viewPtr = (CControlView*)lParam;
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)viewPtr->GetDocument();

	// Complain if training images are not extracted or trained
	if (!viewPtr->isTrained || !viewPtr->isExtracted)
	{	AfxMessageBox(RETRAIN_MESSAGE);
		return 1;
	}

	// Display progress dialog box
	viewPtr->pDlg->ShowWindow(SW_SHOWNORMAL);
	viewPtr->pDlg->setText(IDC_PROGRESS_STATIC, 
		CString(RECON_PROG_MSG) + CWinUtil::GetSubPath(doc->iImage));

	// Used to control progress of training process
	viewPtr->stopFlag = FALSE;

	// Get edge detection values
	double sigma = viewPtr->edgeDetectDialog->getDoubleParam();
	int threshold = viewPtr->edgeDetectDialog->getIntParam();

	CImageMatrix image;
	CImageMatrix features;

	/* Object used to reconstruct (age) input image by using
	   neural net for feature extraction. */

	CReconstruct reconstructor(&doc->imageMap);

	/* Check to see if program (neural net) was able to 
	   perform feature extraction.  If not then user must
	   supply feature extraction data. */
	if (viewPtr->isSelected)
	{	if (doc->iParam.allSelected())
		{	reconstructor.reconstruct(doc->iImage, doc->iParam.rect, 
				doc->iParam.angle, doc->imageSpace, doc->eigenFeat, 
				image, features, viewPtr->noExtraction, sigma, 
				&threshold, &viewPtr->stopFlag, setProgress, 0, 100);	
			if (viewPtr->displayStopStatus())
				return 1;
		}
		// User did not select all required features
		else
		{	AfxMessageBox(RECONSTRUCT_MSG1);
			viewPtr->pDlg->ShowWindow(SW_HIDE);
			return 1;	
		}
	}
	else
	{	BOOL flag = reconstructor.reconstruct(doc->iImage, doc->imageSpace, 
			doc->eigenFeat, image, features, doc->iParam, viewPtr->noExtraction, 
			sigma, &threshold, &viewPtr->stopFlag, setProgress, 0, 100);
		if (viewPtr->displayStopStatus())
			return 1;
		if (!flag)
		{	AfxMessageBox(RECONSTRUCT_MSG2);
			viewPtr->pDlg->ShowWindow(SW_HIDE);
			return 1;	
		}
	}

	// Get values for low pass filter
	double cutoff = viewPtr->filterDialog->getDoubleParam();
	int filterSize = viewPtr->filterDialog->getIntParam();

	// Filter reconstructed image (low pass filter for smoothing and blending)
	image.filterImage(viewPtr->filteredImage, filterSize, cutoff);
	features.filterImage(viewPtr->filteredFeatures, filterSize, cutoff);

	// Set border text to reflect current displayed image name
	CView* view = (COutputView*)doc->getView(RUNTIME_CLASS(COutputView));
	((COutputView*)view)->setBorderText(RECON_IMAGE_MSG, 
		CWinUtil::GetSubPath(doc->iImage));

	// Display reconstructed/filtered image in output view
	if (viewPtr->featureCheckBox.GetCheck() != BST_CHECKED)
		((COutputView*)view)->display(viewPtr->filteredImage);
	else
		((COutputView*)view)->display(viewPtr->filteredFeatures);
	view = doc->getView(RUNTIME_CLASS(CInputView));

	/* Checks to see if input image edge map checkbox is checked.  If so,
	   display edge detected input image.  In either case, display
	   input image with correct in-plane rotation applied. */
	if (viewPtr->isInputChecked())
		((CInputView*)view)->displayEdges(doc->iImage, sigma, &threshold);
	else
		((CInputView*)view)->displayRotate(doc->iImage);	

	// Display in-plane rotation angle
	CString angleText;
	angleText.Format("%.2f", doc->iParam.angle * RAD_TO_DEG_CONV);
	SetDlgItemText(viewPtr->m_hWnd, IDC_INPUT_ANGLE_EDIT, angleText);
	setProgress(0, 0);
	viewPtr->featureCheckBox.EnableWindow();
	viewPtr->pDlg->ShowWindow(SW_HIDE);
	return 0;
}

void CControlView::OnBnClickedReconstructButton()
{
	reconstruct();
}

void CControlView::reconstruct()
{
	AfxBeginThread(reconstructThread, this);
}

/* Display selected training image in training view by 
   double-clicking corresponding training image path 
   in listbox. */
void CControlView::OnNMDblclkTrainingList(NMHDR *pNMHDR, LRESULT *pResult)
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CButton* checkBox = (CButton*)GetDlgItem(IDC_TRAIN_EDGE_CHECK);

	/* Get index of user-selected item in training image
	   listbox. */
	int i = trainingList.GetNextItem(-1, LVNI_SELECTED);
	if (i == -1)
		return;

	/* Reset data corresponding to currently displayed training
	   image. */
	((CPCA_ReconstructorDoc*)GetDocument())->tParam.reset();

	SetDlgItemText(IDC_ANGLE_EDIT, ZERO_TEXT);
	doc->dImage = trainingList.GetItemText(i, 0);
	double angle = doc->imageMap.getAngle(doc->dImage);
	CString angleText;
	angleText.Format("%.2f", angle * RAD_TO_DEG_CONV);
	SetDlgItemText(IDC_ANGLE_EDIT, angleText);
	CTrainingView* view = (CTrainingView*)doc->getView(
		RUNTIME_CLASS(CTrainingView));
	view->setBorderText(TRAINING_IMAGE_MSG1, doc->dImage);

	// Get edge detection values
	double sigma = edgeDetectDialog->getDoubleParam();
	int threshold = edgeDetectDialog->getIntParam();

	// Obtains parameters to correctly display selected training image
	view->updateTrainingParams(doc->dImage);

	// Either displays training image in edge detected format or not
	if (checkBox->GetCheck() == BST_CHECKED)
		view->displayEdges(doc->dImage, sigma, &threshold);	
	else
		view->displayRotate(doc->dImage);

	// Enable appropriate controls
	GetDlgItem(IDC_ANGLE_SPIN)->EnableWindow();
	GetDlgItem(IDC_TRAIN_EDGE_CHECK)->EnableWindow();
	*pResult = 0;
}

// Thread for feature extraction and eigenface training
UINT trainingThread(LPVOID lParam)
{
	CControlView* view = (CControlView*)lParam;
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)view->GetDocument();
	view->isRunning = TRUE;
	view->stopFlag = FALSE;

	// Show progress dialog box
	view->pDlg->ShowWindow(SW_SHOWNORMAL);

	// Obtain edge detection parameters
	double sigma = view->edgeDetectDialog->getDoubleParam();
	int threshold = view->edgeDetectDialog->getIntParam();

	/* If training images have already been feature-extracted, then
	   just train image classifier. */
	if (!view->isExtracted)
	{	view->pDlg->setText(IDC_PROGRESS_STATIC, EXTRACT_PROG_MSG);

		// Use neural net to locate face features (e.g. eyes, face)
		doc->imageMap.extractFeatureData(sigma, &threshold, 
			&view->stopFlag, setProgress);

		// Check if user stopped progress
		if (view->displayStopStatus())
		{	view->isRunning = FALSE;
			return 1;
		}
		view->isExtracted = TRUE;
	}
	view->pDlg->setText(IDC_PROGRESS_STATIC, TRAINING_PROG_MSG);

	int perc = view->pcaTruncationDialog->getTruncPercentage();
	double fraction = (double)perc / 100.0;

	// Compute eigen-eyes, eigen-nose, eigen-mouth, etc
	CEigenUtil eigenUtil(&doc->imageMap);
	eigenUtil.setEigenFeatures(doc->imageSpace, doc->eigenFeat, 
		fraction, view->noExtraction, &view->stopFlag, setProgress);

	if (view->displayStopStatus())
	{	view->isRunning = FALSE;
		return 1;
	}
	view->isTrained = TRUE;
	AfxMessageBox(EXTRACT_TRAIN_MESSAGE);
	view->isRunning = FALSE;
	view->pDlg->ShowWindow(SW_HIDE);

	return 0;
}

void CControlView::OnBnClickedTrainButton()
{
	AfxBeginThread(trainingThread, this);
}

// Rotate displayed training image incrementally
void CControlView::OnDeltaposAngleSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CView* view = doc->getView(RUNTIME_CLASS(CTrainingView));
	CButton* checkBox = (CButton*)GetDlgItem(IDC_TRAIN_EDGE_CHECK);
	doc->tParam.angle += (ANGLE_INC * DEG_TO_RAD_CONV * pNMUpDown->iDelta);
	double sigma = edgeDetectDialog->getDoubleParam();
	int threshold = edgeDetectDialog->getIntParam();
	if (checkBox->GetCheck() == BST_CHECKED)
		((CTrainingView*)view)->displayEdges(doc->dImage, sigma, &threshold);
	else
		((CTrainingView*)view)->displayRotate(doc->dImage);
	CString angleText;
	angleText.Format("%.2f", doc->tParam.angle * RAD_TO_DEG_CONV);
	SetDlgItemText(IDC_ANGLE_EDIT, angleText);
	*pResult = 0;
}

// Rotate displayed input image incrementally
void CControlView::OnDeltaposInputAngleSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CView* view = doc->getView(RUNTIME_CLASS(CInputView));
	CButton* checkBox = (CButton*)GetDlgItem(IDC_INPUT_EDGE_CHECK);
	doc->iParam.angle += (ANGLE_INC * DEG_TO_RAD_CONV * pNMUpDown->iDelta);
	double sigma = edgeDetectDialog->getDoubleParam();
	int threshold = edgeDetectDialog->getIntParam();
	if (checkBox->GetCheck() == BST_CHECKED)
		((CInputView*)view)->displayEdges(doc->iImage, sigma, &threshold);
	else
		((CInputView*)view)->displayRotate(doc->iImage);
	CString angleText;
	angleText.Format("%.2f", doc->iParam.angle * RAD_TO_DEG_CONV);
	SetDlgItemText(IDC_INPUT_ANGLE_EDIT, angleText);
	*pResult = 0;
}

void CControlView::OnEdgeDetectParameters()
{
	edgeDetectDialog->ShowWindow(SW_SHOWNORMAL);
}

void CControlView::OnBnClickedEdgeParamButton()
{
	edgeDetectDialog->ShowWindow(SW_SHOWNORMAL);
}

void CControlView::OnFilterParameters()
{
	filterDialog->ShowWindow(SW_SHOWNORMAL);
}

void CControlView::OnBnClickedFilterParamButton()
{
	filterDialog->ShowWindow(SW_SHOWNORMAL);
}

void CControlView::OnParametersPCAtruncation()
{
	pcaTruncationDialog->ShowWindow(SW_SHOWNORMAL);
}

void CControlView::OnBnClickedPCATruncationButton()
{
	pcaTruncationDialog->ShowWindow(SW_SHOWNORMAL);
}

// Edge detect displayed training image
void CControlView::OnBnClickedTrainEdgeCheck()
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CView* view = doc->getView(RUNTIME_CLASS(CTrainingView));
	CButton* checkBox = (CButton*)GetDlgItem(IDC_TRAIN_EDGE_CHECK);
	double sigma = edgeDetectDialog->getDoubleParam();
	int threshold = edgeDetectDialog->getIntParam();
	if (checkBox->GetCheck() == BST_CHECKED)
		((CTrainingView*)view)->displayEdges(doc->dImage, sigma, &threshold);
	else
		((CTrainingView*)view)->displayRotate(doc->dImage);
}

// Edge detect displayed input image
void CControlView::OnBnClickedInputEdgeCheck()
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CView* view = doc->getView(RUNTIME_CLASS(CInputView));
	CButton* checkBox = (CButton*)GetDlgItem(IDC_INPUT_EDGE_CHECK);
	double sigma = edgeDetectDialog->getDoubleParam();
	int threshold = edgeDetectDialog->getIntParam();
	if (checkBox->GetCheck() == BST_CHECKED)
		((CInputView*)view)->displayEdges(doc->iImage, sigma, &threshold);
	else
		((CInputView*)view)->displayRotate(doc->iImage);
}

// Is training image edge map checkbox checked?
BOOL CControlView::isTrainingChecked() const
{
	return trainCheckBox.GetCheck() == BST_CHECKED ? TRUE : FALSE;
}

// Is input image edge map checkbox checked?
BOOL CControlView::isInputChecked() const
{
	return inputCheckBox.GetCheck() == BST_CHECKED ? TRUE : FALSE;
}

BOOL CControlView::isFeatureOnlyChecked() const
{
	return featureCheckBox.GetCheck() == BST_CHECKED ? TRUE : FALSE;
}

void CControlView::OnImageReconstruct()
{
	reconstruct();
}

UINT batchReconstructThread(LPVOID lParam)
{
	char path[BUFFER];

	CControlView* viewPtr = (CControlView*)lParam;
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)viewPtr->GetDocument();

	/* Open up directory list box to select directory where training
	   images are stored. */
	BOOL flag = CWinUtil::DemoBrowseForFolder(path, BATCH_RECONSTRUCT_DIR_MSG);

	if (!flag)
		return 1;

	CFileFind finder;
	CString filePath(path);

	// Display error dialog if file path is not valid
	if (!finder.FindFile(filePath))
	{	AfxMessageBox(INVALID_PATH_MSG);
		return 1;
	}

	// Complain if training images are not extracted or trained
	if (!viewPtr->isTrained || !viewPtr->isExtracted)
	{	AfxMessageBox(RETRAIN_MESSAGE);
		return 1;
	}

	// Get edge detection values
	double sigma = viewPtr->edgeDetectDialog->getDoubleParam();
	int threshold = viewPtr->edgeDetectDialog->getIntParam();

	// Get values for low pass filter
	double cutoff = viewPtr->filterDialog->getDoubleParam();
	int filterSize = viewPtr->filterDialog->getIntParam();

	CView* view = (COutputView*)doc->getView(RUNTIME_CLASS(COutputView));
	((COutputView*)view)->clear();

	// Display progress dialog box
	viewPtr->pDlg->ShowWindow(SW_SHOWNORMAL);

	// Used to control progress of training process
	viewPtr->stopFlag = FALSE;

	CFaceParams fParams;
	CStringArray DestDir1;
	CWinUtil::BuildFileList(DestDir1, path);
	for (int i = 0; i < DestDir1.GetSize(); i++)
	{	viewPtr->pDlg->setText(IDC_PROGRESS_STATIC, 
			CString(RECON_PROG_MSG) + CWinUtil::GetSubPath(DestDir1[i]));
		
		CImageMatrix image;
		CImageMatrix features;

		/* Object used to reconstruct (age) input image by using
		neural net for feature extraction. */
		CReconstruct reconstructor(&doc->imageMap);
		flag = reconstructor.reconstruct(DestDir1[i], doc->imageSpace, 
			doc->eigenFeat, image, features, fParams, viewPtr->noExtraction, 
			sigma, &threshold, &viewPtr->stopFlag, setProgress, 0, 100);

		// Check to see if user stopped progress
		if (viewPtr->displayStopStatus())
			break;

		/* Check to see if program (neural net) was able to 
		perform feature extraction.  If not then user must
		supply feature extraction data. */
		if (!flag)
			continue;

		CImageMatrix filtered;

		// Filter reconstructed image (low pass filter for smoothing and blending)
		image.filterImage(filtered, filterSize, cutoff);

		((COutputView*)view)->setBorderText(RECON_IMAGE_MSG, 
			CWinUtil::GetSubPath(DestDir1[i]));

		// Display reconstructed/filtered image in output view
		((COutputView*)view)->display(filtered);

		int index = DestDir1[i].ReverseFind(DOT);
		filtered.save(DestDir1[i].Left(index) + BMP_EXT);

		setProgress(0, 0);
	}

	viewPtr->pDlg->ShowWindow(SW_HIDE);
	return 0;
}

void CControlView::OnImageBatchreconstructandsave()
{
	AfxBeginThread(batchReconstructThread, this);
}

void CControlView::OnBnClickedOnlyFeaturesCheck()
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CView* view = doc->getView(RUNTIME_CLASS(COutputView));
	CButton* checkBox = (CButton*)GetDlgItem(IDC_ONLY_FEATURES_CHECK);
	if (checkBox->GetCheck() != BST_CHECKED)
		((COutputView*)view)->display(filteredImage);
	else
		((COutputView*)view)->display(filteredFeatures);
}

void CControlView::OnUpdateDisableFeatureExtraction(CCmdUI *pCmdUI)
{
	if (noExtraction)
		pCmdUI->SetCheck();
	else
		pCmdUI->SetCheck(FALSE);
}

void CControlView::OnDisableFeatureExtraction()
{
	noExtraction = !noExtraction;
}
