#pragma once

#include "afxcmn.h"
#include "afxwin.h"
#include "ParamDialog.h"
#include "InputView.h"
#include "ProgressDialog.h"
#include "PCATruncationDialog.h"


// CControlView form view

class CControlView : public CFormView
{
	friend class CProgressDialog;
	friend class CTrainingView;
	friend class CInputView;
	friend UINT extractThread(LPVOID);
	friend UINT trainingThread(LPVOID);
	friend UINT reconstructThread(LPVOID);
	friend UINT batchReconstructThread(LPVOID);
	friend void setProgress(int);
	DECLARE_DYNCREATE(CControlView)

public:
	void resetTrainingAngle();
	void resetInputAngle();
	void reconstruct();
	void setStopFlag(int);
	void setTrainedFlag(int);
	void setExtractedFlag(int);
	void setSelected(int);
	BOOL isTrainingChecked() const;
	BOOL isInputChecked() const;
	BOOL isFeatureOnlyChecked() const;

protected:
	CControlView();           // protected constructor used by dynamic creation
	virtual ~CControlView();
	void resetTrainingImages();
	void clearInputParameters();
	void positionControls();
	BOOL displayStopStatus();
	void enableControls(BOOL);

public:
	enum { IDD = IDD_CONTROL_VIEW };

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext&) const;
#endif

protected:
	virtual void DoDataExchange(CDataExchange*);    // DDX/DDV support
	virtual void OnInitialUpdate();

protected:
	CImageMatrix filteredImage;
	CImageMatrix filteredFeatures;
	CParamDialog* edgeDetectDialog;
	CParamDialog* filterDialog;
	CProgressDialog* pDlg;
	CPCATruncationDialog* pcaTruncationDialog;
	int isExtracted;
	int isTrained;
	int stopFlag;
	int isRunning;
	int isSelected;
	int noExtraction;
	static HWND wHnd;
	DECLARE_MESSAGE_MAP()

public:
	afx_msg void OnBnClickedLoadTrainingButton();
	afx_msg void OnBnClickedInputImageButton();
	afx_msg void OnBnClickedReconstructButton();
	afx_msg void OnLvnItemchangedTrainingList(NMHDR*, LRESULT*);
	afx_msg void OnNMDblclkTrainingList(NMHDR*, LRESULT*);
	afx_msg void OnBnClickedTrainButton();
	afx_msg void OnDeltaposAngleSpin(NMHDR*, LRESULT*);
	afx_msg void OnDeltaposInputAngleSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnEdgeDetectParameters();
	afx_msg void OnBnClickedEdgeParamButton();
	afx_msg void OnBnClickedTrainEdgeCheck();
	afx_msg void OnBnClickedInputEdgeCheck();
	afx_msg void OnBnClickedFilterParamButton();
	afx_msg void OnFilterParameters();
	afx_msg void OnParametersPCAtruncation();
	afx_msg void OnBnClickedPCATruncationButton();	
	afx_msg void OnImageReconstruct();
	afx_msg void OnImageBatchreconstructandsave();
	afx_msg void OnBnClickedOnlyFeaturesCheck();
	afx_msg void OnUpdateDisableFeatureExtraction(CCmdUI *pCmdUI);
	afx_msg void OnDisableFeatureExtraction();

protected:
	CListCtrl trainingList;
	CButton loadButton;
	CButton trainButton;
	CButton inputButton;
	CButton reconstructButton;
	CEdit trainingEdit;
	CEdit inputEdit;
	CSpinButtonCtrl trainingSpinCtrl;
	CSpinButtonCtrl inputSpinCtrl;
	CStatic trainingGroup;
	CStatic reconstructGroup;
	CStatic trainingStatic;
	CStatic inputStatic;
	CButton trainCheckBox;
	CButton inputCheckBox;
	CButton featureCheckBox;
};


