/*****************************************************************************

PROJECT:		CS 298 Project

FILE:			SelectView.cpp

PURPOSE:		Implements functionality to allow user to select boxes
				in view area.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

*****************************************************************************/

#include "stdafx.h"
#include "PCA_Reconstructor.h"
#include "PCA_ReconstructorDoc.h"
#include "controlview.h"
#include "PromptDialog.h"
#include "SelectView.h"
#include "constants.h"

IMPLEMENT_DYNCREATE(CSelectView, CImageView)

CSelectView::CSelectView() : param(), dragging()
{
}

CSelectView::~CSelectView()
{
}


BEGIN_MESSAGE_MAP(CSelectView, CImageView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()


void CSelectView::OnInitialUpdate()
{
	CImageView::OnInitialUpdate();
}

void CSelectView::OnDraw(CDC* pDC)
{
	CImageView::OnDraw(pDC);

	CBrush brush;
	brush.CreateSolidBrush(CYAN);

	// Draw user selected cyan rectangles
	for (int i = 0; i < FEATURE_NUM; i++)
		pDC->FrameRect(CRect(param->rect[i].left, 
			param->rect[i].top + BORDER_HEIGHT,
			param->rect[i].right, 
			param->rect[i].bottom + BORDER_HEIGHT), &brush);
}

void CSelectView::display(const CImageMatrix& im)
{
	CImageView::display(im);
}

void CSelectView::display(const CString& imPath)
{
	CImageView::display(imPath);
}

void CSelectView::updateTrainingParams(const CString& path)
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	for (int i = 0; i < FEATURE_NUM; i++)
	{	CRect rect;
		doc->imageMap.setFeatureRect(path, rect, (Feature)i);
		param->rect[i] = rect;
		param->select[i] = doc->imageMap.isFeatureSelected(path, (Feature)i);
	}
	param->angle = doc->imageMap.getAngle(path);
}

void CSelectView::updateInputParams(const CString& path)
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	for (int i = 0; i < FEATURE_NUM; i++)
	{	param->rect[i] = doc->iParam.rect[i];
		param->select[i] = doc->iParam.select[i];
	}
	param->angle = doc->iParam.angle;
}

void CSelectView::displayRotate(const CString& path)
{
	rotate(path, param->angle);
}

void CSelectView::displayEdges(const CString& path, 
double sigma, int* threshold, COLORREF color)
{
	CImageMatrix edgeImage;
	edgeImage.loadImage(path);
	edgeImage.edgeDetect(refImage, sigma, threshold, color);
	refImage.rotateImage(image, param->angle);
	update();
}

void CSelectView::clear()
{
	refImage.destroy();
	CImageView::clear();
}

void CSelectView::rotate(const CString& path, double angle)
{
	refImage.loadImage(path);
	refImage.rotateImage(image, angle);
	update();
}

BOOL CSelectView::checkPoint(CPoint pos)
{
	return FALSE;
}

BOOL CSelectView::boundsCheck(const CString& imageFile, CPoint pos)
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CImageMatrix tmpImage;
	if (imageFile.IsEmpty() || !tmpImage.loadImage(imageFile))
		return FALSE;
	int width = tmpImage.getWidth();
	int height = tmpImage.getHeight();
	if (pos.x >= width || pos.y >= height + 
		BORDER_HEIGHT || pos.y <= BORDER_HEIGHT)
		return FALSE;
	return TRUE;
}

void CSelectView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (!checkPoint(point))
		return;
	startPoint = point;
	endPoint = point;
	dragging = TRUE;
	SetCapture();
}

void CSelectView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (dragging)
	{	CClientDC dc(this);
		CSize size(1, 1);
		CRect startRect(startPoint, point);
		CRect endRect(startPoint, endPoint);
		dc.DrawDragRect(startRect, size, endRect, size);
		endPoint = point;
	}
	else
		CScrollView::OnMouseMove(nFlags, point);
}

void CSelectView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();
	CControlView* cView = (CControlView*)doc->getView(RUNTIME_CLASS(CControlView));
	if (dragging)
	{	ReleaseCapture();
		dragging = FALSE;
		if (image.isNull() || !checkPoint(point))
		{	CRect rect(startPoint, point);
			rect.NormalizeRect();
			InvalidateRect(rect);
			return;
		}
		CFeatureSelectPrompt prompt;
		INT_PTR stat = prompt.DoModal();
		if (stat == -1)
			return;
		CPoint startPt(startPoint.x, startPoint.y - BORDER_HEIGHT);
		CPoint endPt(endPoint.x, endPoint.y - BORDER_HEIGHT);
		param->rect[stat].SetRect(startPt, endPt);
		param->select[stat] = TRUE;
		cView->setSelected(TRUE);
		Invalidate();
		UpdateWindow();
	}
	else
		CScrollView::OnLButtonUp(nFlags, point);
}