/********************************************************************************

PROJECT:		CS 298 Project

FILE:			constants.h

PURPOSE:		Various constants needed throughout application.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

********************************************************************************/

#ifndef _CONSTANTS_H_
#define _CONSTANTS_H_

// Enables or disables feature extraction
//#define GLOBAL

typedef void (*FuncPtr)(int, const char*);

enum Feature { LEFT_EYE, RIGHT_EYE, NOSE, MOUTH, FACE };

const double PI = 3.14159265358979323846;
const double SMALL = 0.0001;

// Colors to be used for drawing in device contexts
const COLORREF BLACK = RGB(0, 0, 0);
const COLORREF WHITE = RGB(255, 255, 255);
const COLORREF RED = RGB(255, 0, 0);
const COLORREF GREEN = RGB(0, 255, 0);
const COLORREF BLUE = RGB(0, 0, 255);
const COLORREF YELLOW = RGB(255, 255, 0);
const COLORREF CYAN = RGB(0, 255, 255);
const COLORREF MAGENTA = RGB(255, 0, 255);

const int BUFFER = 1024;

// Border height of gray title bar in each splitter window
const int BORDER_HEIGHT = 25;

/* Used to paint entire client area black for each splitter
   window. */
const int BORDER_LIMIT = 1500;

// Default scroll length
const int DEFAULT_SCROLL = 100;

const char FILTER[] = "*.jpg|*.jpg|24-bit or 256 colors *.bmp|*.bmp|*.gif|*.gif||";

const char* const APP_HEADING = "CS298";
const char* const APP_TITLE = "PCA Age Reconstruction Program";
const char* const TRAINING_LIST_MSG = "Double-click to display";
const char* const EXTRACT_TRAIN_MESSAGE = "Face features are extracted and trained.";
const char* const TRAINING_MESSAGE = "Image classifier is trained.";
const char* const DIRECTORY_DIALOG_MSG = "Select directory with training classes :";
const char* const INVALID_PATH_MSG = "Path name is invalid.";
const char* const INVALID_PATH_CAPTION = "Invalid path name ...";
const char* const UPDATE_FEATURE_MSG = "Feature(s) are updated.";
const char* const UPDATE_FEATURE_CAPTION = "Updated feature(s) ...";
const char* const RECONSTRUCT_MSG1 = "Not all features were selected in input image.";
const char* const RECONSTRUCT_MSG2 = "Program could not locate features, \n\
please manually select.";
const char* const TRAINING_IMAGE_MSG1 = "   Training Image: ";
const char* const INPUT_IMAGE_MSG = "   Input: ";
const char* const RECON_IMAGE_MSG = "   Aged: ";
const char* const EXTRACT_PROG_MSG = "Extracting features from training images ...";
const char* const TRAINING_PROG_MSG = "Training image classifier ...";
const char* const RECON_PROG_MSG = "Aging image ";
const char* const PROG_STOP = "Operation has been stopped.";
const char* const RETRAIN_MESSAGE = "Images must be loaded and (re)trained.";
const char* const RUNNING_MSG = "Still extracting / training ...";
const char* const EDGE_DETECT_DIALOG_TITLE = "Edge detection parameters";
const char* const FILTER_DIALOG_TITLE = "Low pass filter parameters";
const char* const PARAM_EDGE_DIALOG_TEXT1 = "Sigma:";
const char* const PARAM_EDGE_DIALOG_TEXT2 = "Threshold:";
const char* const PARAM_FILTER_DIALOG_TEXT1 = "Cutoff  (0.0 - 0.5):";
const char* const PARAM_FILTER_DIALOG_TEXT2 = "Filter Size  (Overrides cutoff):";
const char* const IMAGE_SAVE_ERROR_MSG = "Output image could not be saved.";
const char* const IMAGE_SAVE_ERROR_CAP = "Image not saved ...";
const char* const BATCH_RECONSTRUCT_DIR_MSG = "Select directory where input images are located.";
const char* const TRAINING_IMAGE_MSG2 = "Creating training image for ";
const char* const EIGEN_MSG1 = "Computed eigenspace for left eye.";
const char* const EIGEN_MSG2 = "Computed eigenspace for right eye.";
const char* const EIGEN_MSG3 = "Computed eigenspace for nose.";
const char* const EIGEN_MSG4 = "Computed eigenspace for mouth.";
const char* const EIGEN_MSG5 = "Computed eigenspace for face.";
const char* const EIGEN_MSG6 = "Computing eigenfeatures ...";
const char* const DEFAULT_EXT = ".jpg";
const char* const BMP_EXT = "_recon.bmp";
const char* const AGED_EXT = "_AGED";
const char* const MSG_STR1 = "Left Eye";
const char* const MSG_STR2 = "Right Eye";
const char* const MSG_STR3 = "Nose";
const char* const MSG_STR4 = "Mouth";
const char* const MSG_STR5 = "Face";
const char* const ZERO_TEXT = "0.00";
const char* const ELLIPSIS = " ...";
const char* const EMPTY_STR = "";

const char ORIGINAL_IMAGE = 'c';
const char AGED_IMAGE = 'e';
const char SEPARATOR = '\\';
const char DOT = '.';

const int BITS_PER_PIXEL_8 = 8;
const int BITS_PER_PIXEL_24 = 24;
const int COLOR_CHANNEL_NUM = 3;
const int FEATURE_NUM = 5;
const int EDGE_THRESHOLD = 0;
const int PIXEL_MAX = 255; 
const int FILTER_SIZE = 0;
const int INIT_MATRIX_SIZE = 10;
const int DEFAULT_TRUNC_PERC = 100;

const double TOLERANCE = 0.000001;
const double DEFAULT_CUTOFF = 0.5;
const double EDGE_SIGMA = 1.2;
const double SCROLL_FACTOR = 1.25;
const double EYE_MARGIN_X = 1.75;
const double EYE_MARGIN_Y = 0.75;
const double NOSE_MARGIN_X = 1.5;
const double ANGLE_INC = 0.25;
const double DEG_TO_RAD_CONV = PI / 180.0;
const double RAD_TO_DEG_CONV = 180.0 / PI;
const double DEFAULT_INTENSITY = 0.75;
const double RGSECTOR = 2.0 * PI / 3.0;
const double GBSECTOR = 2.0 * RGSECTOR;
const double DEG60INRAD = PI / 3.0;

typedef pair<int, int> Pr;
typedef vector<Pr> VecIntPr;
typedef pair<double, double> DPr;
typedef vector<DPr> VecDPr;

#endif