/********************************************************************************

PROJECT:		CS 298 Project

FILE:			convexhull.cpp

PURPOSE:		Computes convex hull using Graham's Scan.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

REMARKS:		This code is taken from Computational Geometry, Joseph
				O'Rourke.  Cambridge University Press, 1998.

********************************************************************************/

#include "stdafx.h"
#include "convexhull.h"

CHullPoint lowest;

int areaSign(const Pr& a, const Pr& b, const Pr& c)
{
	int area = (b.first - a.first) * (c.second - a.second) -
		(c.first - a.first) * (b.second - a.second);
	if (area > 0) 
		return 1;
	else if (area < 0) 
		return -1;
	else                     
		return 0;
}

int compare(const void *tpi, const void *tpj)
{
	int a, x, y;
	CHullPoint* pi;
	CHullPoint* pj;
	pi = (CHullPoint*)tpi;
	pj = (CHullPoint*)tpj;
	a = areaSign(lowest.v, pi->v, pj->v);
	if (a > 0)
		return -1;
	else if (a < 0)
		return 1;
	else 
	{	x = abs(pi->v.first - lowest.v.first) - 
			abs(pj->v.first - lowest.v.first);
			y = abs(pi->v.second - lowest.v.second) - 
			abs(pj->v.second - lowest.v.second);
		if ((x < 0) || (y < 0)) 
		{	pi->remove = TRUE;
			return -1;
		}
		else if ((x > 0) || (y > 0)) 
		{	pj->remove = TRUE;
			return 1;
		}
		else 
		{	if (pi->vnum > pj->vnum)
				pj->remove = TRUE;
			else 
				pi->remove = TRUE;
			return 0;
		}
	}
}

CConvexHull::CConvexHull() : points(0), count(0)
{
}

CConvexHull::~CConvexHull()
{
	delete [] points;
}

bool CConvexHull::setHull(const VecIntPr& pts, VecIntPr& hull)
{
	delete [] points;
	count = (int)pts.size();
	if (pts.size() == 0)
		return false;
	points = new CHullPoint[count];
	for (int i = 0; i < count; i++)
	{	points[i].vnum = i;
		points[i].remove = false;
		points[i].v.first = pts[i].first;
		points[i].v.second = pts[i].second;
	}
	findLowest();
	qsort(&points[1], count - 1, sizeof(CHullPoint), compare);
	squash();
	hull.clear();
	CStackCell* t = grahamScan();
	while (t) 
	{	hull.push_back(Pr(t->p->v.first, t->p->v.second));
		t = t->next;
	}
	return true;
}

void CConvexHull::findLowest()
{
	int m = 0;
	for (int i = 1; i < count; i++)
		if ((points[i].v.second < points[m].v.second) || 
			((points[i].v.second == points[m].v.second) && 
			(points[i].v.first > points[m].v.first))) 
			m = i;
	swapPoints(0, m);
	lowest = points[0];
}

void CConvexHull::swap(int& x, int& y) 
{ 
	int tmp = x; 
	x = y; 
	y = tmp; 
}

void CConvexHull::swapPoints(int i, int j)
{
   swap(points[i].vnum, points[j].vnum);
   swap(points[i].v.first, points[j].v.first);
   swap(points[i].v.second, points[j].v.second);
   swap(points[i].remove, points[j].remove);
}

CStackCell* CConvexHull::pop(CStackCell* s)
{
	CStackCell* top;
	top = s->next;
	delete s;
	s = 0;
	return top;
}

CStackCell* CConvexHull::push(CHullPoint* p, CStackCell* top)
{
	CStackCell* s;
	s = new CStackCell;
	s->p = p;
	s->next = top;
	return s;
}

CStackCell* CConvexHull::grahamScan()
{
	CHullPoint* p1;
	CHullPoint* p2;
	CStackCell* top;
	top = 0;
	top = push(&points[0], top);
	top = push(&points[1], top);
	int i = 2;
	while (i < count) 
	{	if (!top->next) 
		{	cout << "Error" << endl;
			exit(1);
		}
		p1 = top->next->p;
		p2 = top->p;
		if (isLeft(p1->v, p2->v, points[i].v)) 
		{	top = push(&points[i], top);
			i++;
		} 
		else    
			top = pop(top);
	}
	return top;
}

void CConvexHull::squash()
{
	int i = 0, j = 0;
	while (i < count) 
	{	if (!points[i].remove) 
		{	points[j].v.first = points[i].v.first;
			points[j].v.second = points[i].v.second;
			points[j].vnum = points[i].vnum;
			points[j].remove = points[i].remove;
			j++;
		}
		i++;
	}
	count = j;
}

int CConvexHull::crossProduct(const Pr& a, const Pr& b, const Pr& c)
{
	return (b.first - a.first) * (c.second - a.second) - 
		(c.first - a.first) * (b.second - a.second);
}

bool CConvexHull::isLeft(const Pr& a, const Pr& b, const Pr& c)
{ 
	return crossProduct(a, b, c) > 0;
}