/***************************************************************************

PROJECT:		CS 298 Project

FILE:			convexhull.h

PURPOSE:		Computes convex hull using Graham's Scan.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

REMARKS:		This code is taken from Computational Geometry, Joseph
				O'Rourke.  Cambridge University Press, 1998.

***************************************************************************/ 

#ifndef _CONVEX_HULL_H_
#define _CONVEX_HULL_H_

#include "constants.h"

struct CHullPoint
{
	Pr v;
	int vnum;
	int remove;
};

struct CStackCell
{
	CHullPoint* p;
	CStackCell* next;
};

class CConvexHull
{
public:
	CConvexHull();
	~CConvexHull();
	bool setHull(const VecIntPr&, VecIntPr&);

protected:
	CStackCell* pop(CStackCell*);
	CStackCell* push(CHullPoint*, CStackCell*);
	CStackCell* grahamScan();
	void squash();
	void findLowest();
	void swap(int&, int&);
	void swapPoints(int, int);
	bool isLeft(const Pr&, const Pr&, const Pr&);
	int crossProduct(const Pr&, const Pr&, const Pr&);
	int readPoints();

private:
	CConvexHull(const CConvexHull&);
	operator=(const CConvexHull&);
	CHullPoint* points;
	int count;
};

#endif