/********************************************************************************

PROJECT:		CS 298 Project

FILE:			datastructures.cpp

PURPOSE:		Maps training path name to training data.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

********************************************************************************/

#include "stdafx.h"
#include "imagematrix.h"
#include "featurelocate.h"
#include "datastructures.h"
#include "winutil.h"

CFaceParams::CFaceParams()
{
	reset();
}

BOOL CFaceParams::noneSelected() const
{
	for (int i = 0; i < FEATURE_NUM; i++)
		if (select[i])
			return FALSE;
	return TRUE;
}

BOOL CFaceParams::allSelected() const
{
	for (int i = 0; i < FEATURE_NUM; i++)
		if (!select[i])
			return FALSE;
	return TRUE;
}

void CFaceParams::reset()
{
	for (int i = 0; i < FEATURE_NUM; i++)
	{	rect[i].SetRectEmpty();
		select[i] = FALSE;
	}
	angle = 0.0;
}

void CFaceParams::operator=(const CFaceParams& fParams)
{
	for (int i = 0; i < FEATURE_NUM; i++)
	{	rect[i] = fParams.rect[i];
		select[i] = fParams.select[i];
	}
	angle = fParams.angle;
}

CTrainingData::CTrainingData()
{
	uData = new CData();
	aData = new CData();
}

CTrainingData::~CTrainingData()
{
	delete uData;
	delete aData;
}

CFaceMap::CFaceMap()
{
}

CFaceMap::~CFaceMap()
{
	clearAll();
}

void CFaceMap::addDataMapping(const CString& key, 
const CString& uFile, const CString& aFile)
{
	CTrainingData* data = new CTrainingData();
	data->uData->file = CWinUtil::GetSubPath(uFile);
	data->aData->file = CWinUtil::GetSubPath(aFile);
	dataMap.SetAt(key, data);
	strArr.Add(key);
}

void CFaceMap::extractFeatureData(double sigma, int* threshold, 
int* stopFlag, FuncPtr fp)
{
	CString path;
	CTrainingData* data;
	CImageMatrix uIm, aIm;
	CFeatureLocate faceUtil;
	double count = 0.0;
	double step = 100.0 / dataMap.GetSize();
	sort(strArr.GetData(), strArr.GetData() + strArr.GetSize());
	int i = 0;
	while (i < strArr.GetSize() && !(*stopFlag))
	{	dataMap.Lookup(strArr[i], (void*&)data);
		path = strArr[i] + SEPARATOR + data->uData->file;
		uIm.loadImage(path);
		faceUtil.setImage(&uIm);
		faceUtil.setData(data->uData, sigma, threshold);
		path = strArr[i] + SEPARATOR + data->aData->file;
		aIm.loadImage(path);
		faceUtil.setImage(&aIm);
		faceUtil.setData(data->aData, sigma, threshold);
		int index = strArr[i].ReverseFind(SEPARATOR);
		count += step;
		fp((int)count, strArr[i].Mid(index));
		i++;
	}
}

CData* CFaceMap::getData(const CString& path) const
{
	CString key, file;
	CTrainingData* data;
	if (!split(path, key, file) || !dataMap.Lookup(key, (void*&)data))
		return 0;
	if (file.Compare(data->uData->file) == 0)
		return data->uData;
	else if (file.Compare(data->aData->file) == 0)
		return data->aData;
	else
		return 0;
}

BOOL CFaceMap::split(const CString& path, CString& key, CString& file) const
{
	int index = path.ReverseFind(SEPARATOR);
	if (index == -1)
		return FALSE;
	key = path.Left(index);
	file = path.Right(path.GetLength() - index - 1);
	return TRUE;
}

void CFaceMap::clearAll()
{
	CString nKey;
	CTrainingData* data;
	POSITION pos = dataMap.GetStartPosition();
	while (pos != NULL)
	{	dataMap.GetNextAssoc(pos, nKey, (void*&)data);
		delete data;
	}
	strArr.RemoveAll();
	dataMap.RemoveAll();
}

void CFaceMap::setFeatureRect(const CString& path, CRect& rect, 
Feature feature, BOOL flag)
{
	CData* data = getData(path);
	if (flag)
		data->param.rect[feature] = rect;
	else
		rect = data->param.rect[feature];
}

void CFaceMap::setFaceAngle(const CString& path, double& angle, BOOL flag)
{
	CData* data = getData(path);
	if (flag)
		data->param.angle = angle;
	else
		angle = data->param.angle;
}

BOOL CFaceMap::isFeatureSelected(const CString& path, Feature feature) const
{
	return getData(path)->param.select[feature];
}

void CFaceMap::selectFeature(const CString& path, Feature feature, BOOL flag)
{
	getData(path)->param.select[feature] = flag;
}

int CFaceMap::getWidth(const CString& path) const
{
	return getData(path)->width;
}

int CFaceMap::getHeight(const CString& path) const
{
	return getData(path)->height;
}

double CFaceMap::getAngle(const CString& path) const
{
	return getData(path)->param.angle;
}

int CFaceMap::getSize() const
{
	return (int)dataMap.GetSize();
}

BOOL CFaceMap::isPathValid(const CString& path) const
{
	return getData(path) != 0 ? TRUE : FALSE;
}
