/***************************************************************************

PROJECT:		CS 298 Project

FILE:			edgedetector.h

PURPOSE:		Function declarations for Boie-Cox edge detector to
				perform edge detection on a grayscale image.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

REMARKS:		This code is taken from
				http://www.ee.ucl.ac.uk/~icox/#Anchor--Ima-25529.
				The code was originally written in C, but was
				converted into slightly more object-oriented 
				C++ code.

***************************************************************************/

#ifndef _EDGE_FINDER_H_
#define _EDGE_FINDER_H_

class CEdgeDetector
{
public:
	CEdgeDetector(unsigned char* = 0, int = 0, int = 0);
	~CEdgeDetector();
	void freeImage();
	void setImage(unsigned char*, int, int);
	unsigned char* edge_detect(double, int*);
	unsigned char* image_find_edges(double, int*);
	unsigned char* image_hysteresis(int);
	unsigned char* image_thin();
	unsigned char* get_image() const { return an_image; }
	unsigned char* get_edge_map() const { return edge_map; }
	unsigned char* get_edge_map_high() const { return edge_map_hi; }
	int get_x() const { return nx; }
	int get_y() const { return ny; }

protected:
	void image_detx();
	void image_dety();
	void image_det45();
	void image_det135();
	void generate_filter(double);
	void image_convolve(unsigned char*);
	void image_neighbor_fwd(int, int, unsigned char*);
	void image_neighbor_bwd(int, int, unsigned char*);
	unsigned char* image_edges(int);
	int image_locx(int, int, int);
	int image_locy(int, int, int);
	int image_loc45(int, int, int);
	int image_loc135(int, int, int);
	int image_zc_x(int, int, int);
	int image_zc_y(int, int, int);
	int image_zc_135(int, int, int);
	int image_zc_45(int, int, int);
	int image_edge_map_lo(int, int, int);
	int Fcontour(int, int);
	int Bcontour(int, int);
	int image_threshold();

private:
	unsigned char *edge_map, *edge_map_hi, *an_image;
	int nx, ny, norm, nf, nf2, orient_flag, lo_threshold;
	int *gaussian, *idx, *idy, *id45, *id135, *filter;
};

#endif