/**************************************************************************

PROJECT:		CS 298 Project

FILE:			eigenutil.cpp

PURPOSE:		Code that contains the numerical computation
				algorithms to calculate eigenvectors and
				eigenvalues of a symmetric matrix, used
				to compute eigenspaces.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

COMMENTS:		Jacobi code to solve for eigenvectors and eigenvalues 
				is taken from "Numerical Recipes in 
				C", by William H. Press et al., 1989; code modified 
				to 0-base index.

**************************************************************************/ 

#include "stdafx.h"
#include "imagematrix.h"
#include "eigenutil.h"

CEigenParams::CEigenParams()
{
	for (int i = 0; i < FEATURE_NUM; i++)
		num[i] = width[i] = height[i] = 0;
}

CEigenUtil::CEigenUtil(CFaceMap* mp) : faceMap(mp)
{
}

CEigenUtil::~CEigenUtil()
{
}

void CEigenUtil::setEigenFeatures(CMatrix* imgSpc, CMatrix* eSpc, 
double fraction, int noExtraction, int* stopFlag, FuncPtr fp)
{
	if (faceMap == 0)
		return;
	double count = 0.0;
	double step = 100.0 / (faceMap->getSize() * 2.0);
	CMatrix eVal[FEATURE_NUM];
	CMatrix eVec[FEATURE_NUM];
	int indices[FEATURE_NUM] = { 0 };
	CEigenParams eParam;
	setEigenParams(eParam);
	int i, j, n;
	for (i = 0; i < FEATURE_NUM; i++)
		imgSpc[i].SetSize(eParam.num[i], (eParam.height[i] + 1) *
			(eParam.width[i] + 1) * COLOR_CHANNEL_NUM * 2);
	i = 0;
	while (i < faceMap->strArr.GetSize())
	{	CTrainingData* data;	
		faceMap->dataMap.Lookup(faceMap->strArr[i], (void*&)data);
		CString uPath = faceMap->strArr[i] + SEPARATOR + data->uData->file;
		CString aPath = faceMap->strArr[i] + SEPARATOR + data->aData->file;
		CFaceParams* uParam = &data->uData->param;
		CFaceParams* aParam = &data->aData->param;

		if (noExtraction)
			n = FEATURE_NUM - 1;
		else
			n = 0;
		for (j = FEATURE_NUM - 1; j >= n; j--)
		{	if (*stopFlag)
				return;
			if (uParam->select[j] && aParam->select[j])
				setTrainingData(uPath, aPath, uParam, aParam, 
					eParam, imgSpc, j, indices[j]++);
		}
		count += step;
		int index = faceMap->strArr[i].ReverseFind(SEPARATOR);
		fp((int)count, CString(TRAINING_IMAGE_MSG2) + faceMap->strArr[i].Mid(index));
		i++;
	}
	fp((int)count, EIGEN_MSG6);
	count = 50.0;
	if (noExtraction)
	{	n = FEATURE_NUM - 1;
		step = count;
	}
	else
	{	step = count / FEATURE_NUM;
		n = 0;
	}
	for (i = FEATURE_NUM - 1; i >= n; i--)
	{	CMatrix mean = imgSpc[i].meanSubtracted();
		eigenSolve(mean.getSymmetric(), eVal[i], eVec[i]);
		int rowNo = (int)eVec[i].RowNo();
		int colNo = (int)(fraction * eVec[i].ColNo());
		eVec[i].SetSize(rowNo, colNo);
		eSpc[i].SetSize(eVec[i].ColNo(), imgSpc[i].ColNo());
		setEigenSpace(eVec[i], mean, eSpc[i], stopFlag, fp);
		if (*stopFlag)
			return;
		count += step;
		if (i == 0)
			fp((int)count, EIGEN_MSG1);
		else if (i == 1)
			fp((int)count, EIGEN_MSG2);
		else if (i == 2)
			fp((int)count, EIGEN_MSG3);
		else if (i == 3)
			fp((int)count, EIGEN_MSG4);
		else if (i == 4)
			fp((int)count, EIGEN_MSG5);
	}
}

void CEigenUtil::setEigenSpace(const CMatrix& eVec, 
const CMatrix& mean, CMatrix& eSpc, int* stopFlag, FuncPtr fp)
{
	int eVecWidth = (int)eVec.ColNo();
	int eVecHeight = (int)eVec.RowNo();
	int meanWidth = (int)mean.ColNo();
	double** eVecP = eVec.getMatrix();
	double** meanP = mean.getMatrix();
	double** eSpcP = eSpc.getMatrix();
	double* array = new double[meanWidth];
	for (int i = 0; i < eVecWidth; i++)
	{	double norm = 0.0;
		for (int j = 0; j < meanWidth; j++)
		{	if (*stopFlag)
				return;	
			array[j] = 0.0;
			for (int k = 0; k < eVecHeight; k++)
				array[j] += (eVecP[k][i] * meanP[k][j]);
			norm += (array[j] * array[j]);
		}
		for (int m = 0; m < meanWidth; m++)
			eSpcP[i][m] = array[m] / sqrt(norm);
	}
	delete [] array;
}

void CEigenUtil::setTrainingData(const CString& uPath, 
const CString& aPath, CFaceParams* uParam, 
CFaceParams* aParam, CEigenParams& eParam, 
CMatrix* m, int index, int i)
{
	CMatrix out;
	int width = eParam.width[index];
	int height = eParam.height[index];
	CImageMatrix uImIn, uImOut, aImIn, aImOut;
	uImIn.loadImage(uPath);
	uImIn.rotateImage(uImOut, uParam->angle);
	uImIn.setImage(uImOut, uParam->rect[index]);
	double xs = (double)(width + 1) / (uParam->rect[index].Width() + 1);
	double ys = (double)(height + 1) / (uParam->rect[index].Height() + 1);
	uImIn.scaleImage(uImOut, xs, ys);
	aImIn.loadImage(aPath);
	aImIn.rotateImage(aImOut, aParam->angle);
	aImIn.setImage(aImOut, aParam->rect[index]);
	xs = (double)(width + 1) / (aParam->rect[index].Width() + 1);
	ys = (double)(height + 1) / (aParam->rect[index].Height() + 1);
	aImIn.scaleImage(aImOut, xs, ys);
	CImageMatrix::demultiplex(uImOut, aImOut, out);
	m[index].setRow(i, out);
}

void CEigenUtil::setEigenParams(CEigenParams& eParam) const
{
	CString key;
	CTrainingData* data;
	POSITION pos = faceMap->dataMap.GetStartPosition();
	while (pos != 0)
	{	faceMap->dataMap.GetNextAssoc(pos, key, (void*&)data);
		CFaceParams* uParam = &data->uData->param;
		CFaceParams* aParam = &data->aData->param;
		for (int i = 0; i < FEATURE_NUM; i++)
		{	if (uParam->select[i] && aParam->select[i])
				eParam.num[i]++;
			if (uParam->rect[i].Width() > eParam.width[i])
				eParam.width[i] = uParam->rect[i].Width();
			if (uParam->rect[i].Height() > eParam.height[i])
				eParam.height[i] = uParam->rect[i].Height();
			if (aParam->rect[i].Width() > eParam.width[i])
				eParam.width[i] = aParam->rect[i].Width();
			if (aParam->rect[i].Height() > eParam.height[i])
				eParam.height[i] = aParam->rect[i].Height();
		}
	}
}

void CEigenUtil::eigenSolve(CMatrix& m, CMatrix& eVal, CMatrix& eVec)
/* PURPOSE:     Perform Jacobian transformation on symmetric CMatrix object
                "m" and store eigenvalues and eigenvectors in CMatrix objects.

   RECEIVES:    CMatrix object "m" to diagonalize, and CMatrix objects
                to store eigenvalues and eigenvectors, respectively.
*/
{
	eVal.SetSize(1, m.ColNo());
	eVec.SetSize(m.RowNo(), m.ColNo());
	jacobi(m, eVal, eVec);
}

void CEigenUtil::jacobi(CMatrix& m, CMatrix& eVal, CMatrix& eVec)
/* PURPOSE:     Performs Jacobian transformation on CMatrix "symmetric"
                towards a diagonal matrix, where eigenvalues and eigenvectors
                are stored in CMatrix objects.

   RECEIVES:    Symmetric matrix "symmetric" to be diagonalized, and
                CMatrix objects "eigenvalues" and "eigenvectors".

   REMARKS:     1) Contents CMatrix "symmetric" are destroyed.
                
                2) Borrowed from "Numerical Recipes in C"

                3) Modified to 0-based array indexing.
*/
{
	int i, j, iq, ip;
	double tresh, theta, tau, t, sm, s, h, g, c;
	int n = (int)m.RowNo();
	double* b = new double[n];
	double* z = new double[n];
	for (ip = 0; ip < n; ip++)
	{	for (iq = 0; iq < n; iq++)
			eVec(ip, iq) = 0.0;
		eVec(ip, ip) = 1.0;
	}
	for (ip = 0; ip < n; ip++)
	{	b[ip] = eVal(0, ip) = m(ip, ip);
		z[ip] = 0.0;
	}
	for (i = 1; i <= 50; i++)
	{	sm = 0.0;
		for (ip = 0; ip < n - 1; ip++)
		{	for (iq = ip + 1; iq < n; iq++)
				sm += fabs(m(ip, iq));
		}
		if (sm == 0.0)
		{	delete [] z;
			delete [] b;
			return;
		}
		if (i < 4)
			tresh = 0.2 * sm / (n * n);
		else
			tresh = 0.0;
		for (ip = 0; ip < n - 1; ip++)
		{	for (iq = ip + 1; iq < n; iq++)
			{	g = 100.0 * fabs(m(ip, iq));
				if (i > 4 && (double)(fabs(eVal(0, ip)) + g) ==
					(double)fabs(eVal(0, ip)) && 
					(double)(fabs(eVal(0, iq)) + g) == 
					(double)fabs(eVal(0, iq)))
					m(ip, iq) = 0.0;
				else if (fabs(m(ip, iq)) > tresh)
				{	h = eVal(0, iq) - eVal(0, ip);
					if ((double)(fabs(h) + g) == (double)fabs(h))
						t = (m(ip, iq)) / h;
					else
					{	theta = 0.5 * h / (m(ip, iq));
						t = 1.0 / (fabs(theta) + sqrt(1.0 + theta * theta));
						if (theta < 0.0)
							t = -t;
					}
					c = 1.0 / sqrt(1 + t * t);
					s = t * c;
					tau = s / (1.0 + c);
					h = t * m(ip, iq);
					z[ip] -= h;
					z[iq] += h;
					eVal(0, ip) -= h;
					eVal(0, iq) += h;
					m(ip, iq) = 0.0;
					for (j = 0; j <= ip - 1; j++)
						rotate(m, j, ip, j, iq, s, tau);
					for (j = ip + 1; j <= iq - 1; j++)
						rotate(m, ip, j, j, iq, s, tau);
					for (j = iq + 1; j < n; j++)
						rotate(m, ip, j, iq, j, s, tau);
					for (j = 0; j < n; j++)
						rotate(eVec, j, ip, j, iq, s, tau);
				}
			}
		}
		for (ip = 0; ip < n; ip++)
		{	b[ip] += z[ip];
			eVal(0, ip) = b[ip];
			z[ip] = 0.0;
		}
	}
}

void CEigenUtil::rotate(CMatrix& a, int i, int j, int k, int l, double s, double tau)
/* PURPOSE:     Perform plane rotation on symmetric matrix "a" towards
                a diagonal matrix.

   RECEIVES:    Parameters as needed by function "jacobi" below.

   REMARKS:     Borrowed from "Numerical Recipes in C" and modified
                   from a macro to a function.
*/
{
	double g = a(i, j);
	double h = a(k, l);
	a(i, j) = g - s * (h + g * tau);
	a(k, l) = h + s * (g - h * tau);
}