/*************************************************************************

PROJECT:		CS 298 Project

FILE:			eigenutil.h

PURPOSE:		Declaration for function that performs the numerical
				computations to calculate eigenvectors and eigenvalues
				of a symmetric matrix, used to compute eigenspaces.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

REMARKS:		Jacobi code to solve for eigenvectors and eigenvalues 
				is taken from "Numerical Recipes in 
				C", by William H. Press et al., 1989; code modified 
				to 0-base index.

*************************************************************************/ 

#ifndef _EIGENSOLVE_H_
#define _EIGENSOLVE_H_

#include "datastructures.h"

struct CEigenParams
{
	CEigenParams();
	int num[FEATURE_NUM];
	int width[FEATURE_NUM];
	int height[FEATURE_NUM];
};

class CEigenUtil
{
public:
	CEigenUtil(CFaceMap* = 0);
	~CEigenUtil();
	void setEigenFeatures(CMatrix*, CMatrix*, 
		double, int, int*, FuncPtr);
	void setEigenParams(CEigenParams&) const;

private:
	CFaceMap* faceMap;
	void setTrainingData(const CString&, const CString&, 
		CFaceParams*, CFaceParams*, CEigenParams&, CMatrix*, int, int);
	void eigenSolve(CMatrix&, CMatrix&, CMatrix&);
	void jacobi(CMatrix&, CMatrix&, CMatrix&);
	void eigenSort(CMatrix&, CMatrix&);
	void rotate(CMatrix&, int, int, int, int, double, double);
	void setEigenSpace(const CMatrix&, const CMatrix&, 
		CMatrix&, int*, FuncPtr);
};

#endif