/***************************************************************************

PROJECT:		CS 298 Project

FILE:			imagematrix.h

PURPOSE:		Implements image scaling, rotation, filtering, mirroring, etc.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05

REMARKS:		Code for image rotation, scaling, and filtering are taken
				from Practical Algorithms for Image Analysis: Description, 
				Examples, and Code.  Michael Seul, Lawrence O'Gorman, 
				and Michael J. Sammon. Cambridge University Press, 2000.

***************************************************************************/ 

#ifndef _IMAGE_SUPPORT_H_
#define _IMAGE_SUPPORT_H_

#include "matrx.h"
#include "convexhull.h"
using namespace Gdiplus;

enum ColorType { Red, Green, Blue };

class CImageMatrix
{
public:
	CImageMatrix();
	~CImageMatrix();
	BOOL createRGBImage(int, int, COLORREF = BLACK);
	BOOL loadImage(const CString&);
	BYTE* getGrayBuffer() const;
	BYTE* getGrayBuffer(const CRect&) const;
	BYTE* getColorBuffer(ColorType) const;
	BOOL setImage(const CImageMatrix&);
	BOOL setImage(const CImageMatrix&, const CRect&);
	void overlay(const CPoint&, const CImageMatrix&, BOOL = FALSE);
	void colorRegion(const CRect&, COLORREF);
	BOOL edgeDetect(CImageMatrix&, double, int*, COLORREF);
	BOOL rotateImage(CImageMatrix&, double);
	BOOL scaleImage(CImageMatrix&, double, double);
	BOOL filterImage(CImageMatrix&, int = FILTER_SIZE, 
		double = DEFAULT_CUTOFF);
	BOOL isolateFeature(CImageMatrix&, double, int*);
	BOOL isNull() const;
	HBITMAP getHBitmap() const;
	int getWidth() const;
	int getHeight() const;
	void destroy();
	BOOL save(const char*, REFGUID = GUID_NULL);
	static BOOL demultiplex(const CImageMatrix&, 
		const CImageMatrix&, CMatrix&);
	static BOOL multiplex(const CMatrix&, 
		CImageMatrix&, CImageMatrix&);

protected:
	BOOL setImage(const CImageMatrix&, 
		int, int, int, int, int, int);
	void scaleDim(double, double, int&, int&);
	void rotateRGBImage(CImageMatrix&, double);
	void rotate8BitImage(CImageMatrix&, double);
	void scaleRGBImage(CImageMatrix&, double, double);
	void scale8BitImage(CImageMatrix&, double, double);
	void lowPassFilter(BYTE*, int, int, int, double);
	void convolve(BYTE*, int, int, int*, int);

private:	
	CImageMatrix(const CImageMatrix&);
	operator=(const CImageMatrix&);
	static CConvexHull convexHull;
	CImage image;
};

#endif