/********************************************************************************

PROJECT:		CS 298 Project

FILE:			imageview.cpp

PURPOSE:		Implements functionality to display images.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

********************************************************************************/

#include "stdafx.h"
#include "PCA_Reconstructor.h"
#include "PCA_ReconstructorDoc.h"
#include "imageview.h"
#include "constants.h"

IMPLEMENT_DYNCREATE(CImageView, CScrollView)

CImageView::CImageView()
{
}

CImageView::~CImageView()
{
}

BEGIN_MESSAGE_MAP(CImageView, CScrollView)
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

void CImageView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = DEFAULT_SCROLL;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

BOOL CImageView::OnEraseBkgnd(CDC*)
{
	return FALSE;
}

void CImageView::OnDraw(CDC* dc)
{
	CPCA_ReconstructorDoc* doc = (CPCA_ReconstructorDoc*)GetDocument();

	CDoubleBuffer pDC(dc);

	CRect rect, rcClient;

	// Get client window area in pixel dimensions
	GetClientRect(rcClient);

	// Get system defined colors
	COLORREF crLight = GetSysColor(COLOR_BTNHIGHLIGHT);
	COLORREF crShadow = GetSysColor(COLOR_BTNSHADOW);
	COLORREF crBtnFace = GetSysColor(COLOR_BTNFACE);
	pDC->SetBkMode(TRANSPARENT);
	CGdiObject *pOldFont = pDC->SelectStockObject(ANSI_VAR_FONT);
	rect = rcClient;

	// Draw title border and set title text
	rect.bottom = rect.top + BORDER_HEIGHT;
	rect.right = BORDER_LIMIT;
	pDC->FillSolidRect(rect, crBtnFace);
	pDC->Draw3dRect(rect, crLight, crShadow);
	pDC->DrawText(borderText, rect, DT_SINGLELINE | DT_VCENTER);

	// Fill client area as black
	rect.top = rect.bottom;
	rect.bottom = BORDER_LIMIT;
	pDC->FillSolidRect(rect, BLACK);
	pDC->SelectObject(pOldFont);

	if (image.isNull())
		return;

	int width = image.getWidth();
	int height = image.getHeight();
	
	// Create compatible bitmap to draw on
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap(pDC, width, height);

	// Create compatible device context to blit from
	CDC cdc;
	cdc.CreateCompatibleDC(pDC);

	// Assign image to device context
	cdc.SelectObject(CBitmap::FromHandle(image.getHBitmap()));

	// Display image
	pDC->BitBlt(0, BORDER_HEIGHT, width, height, &cdc, 0, 0, SRCCOPY);
}

void CImageView::setBorderText(const CString& text, const CString& name)
{
	int index = name.ReverseFind(SEPARATOR);
	index = name.Left(index).ReverseFind(SEPARATOR);
	borderText = text + ELLIPSIS + name.Right(name.GetLength() - index);
}

void CImageView::display(const CString& imPath)
{
	CImageMatrix im;
	im.loadImage(imPath);
	display(im);
}

void CImageView::display(const CImageMatrix& im)
{
	image.setImage(im);
	update();
}

void CImageView::clear()
{
	image.destroy();
	borderText.Empty();
	update();
}

void CImageView::update()
{	
	Invalidate();
	UpdateWindow();
	scroll();
}

void CImageView::scroll()
{	
	if (image.isNull())
		return;
	CSize sizeTotal;
	sizeTotal.cx = (long)(SCROLL_FACTOR * image.getWidth());
	sizeTotal.cy = (long)(SCROLL_FACTOR * image.getHeight());
	SetScrollSizes(MM_TEXT, sizeTotal);
}

#ifdef _DEBUG
void CImageView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CImageView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif