/***************************************************************************

PROJECT:		CS 298 Project

FILE:			imageview.h

PURPOSE:		Implements functionality to display images.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

***************************************************************************/ 

#pragma once

#include "imagematrix.h"

class CDoubleBuffer : public CDC
{
public:
	CDoubleBuffer(CDC* pDC_) : pDC(pDC_)
	{
		pDC->GetClipBox(&rect);
		CreateCompatibleDC(pDC);
		pDC->LPtoDP(&rect);
		bitmap.CreateCompatibleBitmap(pDC, rect.Width(), rect.Height());
		oldBitmap = SelectObject(&bitmap);
		SetMapMode(pDC->GetMapMode());
		SetWindowExt(pDC->GetWindowExt());
		SetViewportExt(pDC->GetViewportExt());
		pDC->DPtoLP(&rect);
		SetWindowOrg(rect.left, rect.top);
		FillSolidRect(rect, pDC->GetBkColor());
	}

	~CDoubleBuffer()      
	{          
		pDC->BitBlt(rect.left, rect.top, rect.Width(), rect.Height(), 
			this, rect.left, rect.top, SRCCOPY);            
		SelectObject(oldBitmap);          
	}

	CDoubleBuffer* operator->() 
	{
		return this;
	}       

	operator CDoubleBuffer*() 
	{
		return this;
	}

private:       
	CBitmap bitmap;
	CBitmap* oldBitmap;
	CDC* pDC;
	CRect rect;
};

class CImageView : public CScrollView
{
	DECLARE_DYNCREATE(CImageView)

protected:
	CImageView();
	virtual ~CImageView();

protected:
	CImageMatrix image;
	CString borderText;

public:
	virtual void display(const CString&);
	virtual void display(const CImageMatrix&);
	virtual void clear();
	virtual void setBorderText(const CString&, const CString&);
	void update();
	void scroll();

public:
	virtual void OnDraw(CDC*);
	virtual void OnInitialUpdate();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext&) const;
#endif

protected:
	DECLARE_MESSAGE_MAP()
	afx_msg BOOL OnEraseBkgnd(CDC*);
};