/***************************************************************************

PROJECT:		CS 297 Deliverable 1

FILE:			matrx.h

PURPOSE:		Class and implementation that supports most matrix
				operations and calculations.

COMPILER:		Visual C++ .NET Framework 1.0

PROGRAMMER:		Wallun Chan

START DATE:		3/7/05

REMARKS:		This code is taken from website:
				http://www.techsoftpl.com/matrix.

***************************************************************************/ 

#ifndef __cplusplus
#error Must use C++ for the type matrx.
#endif

#if !defined(__STD_MATRIX_H)
#define __STD_MATRIX_H

#if defined(__BORLANDC__)
#pragma option -w-inl -w-pch
#endif

#if (defined(__BORLANDC__) || _MSC_VER <= 1000) && !defined(__GNUG__)
#  include <stdio.h>
#  include <stdlib.h>
#  include <math.h>
#  include <iostream.h>
#  include <string.h>
#else
#  include <cmath>
#  include <cstdio>
#  include <cstdlib>
#  include <string>
#  include <iostream>
#endif

#if defined(_MSC_VER) && _MSC_VER <= 1000
#  define _NO_EXCEPTION
typedef int bool;
#  if !defined(false)
#    define false  0
#  endif
#  if !defined(true)
#    define true   1
#  endif
#endif

#if defined(__BORLANDC__) && !defined(__WIN32__)
#  define _NO_EXCEPTION
#  define _NO_NAMESPACE
#endif

#if defined(_MSC_VER) && !defined(_WIN32)
#  define _NO_EXCEPTION
#endif

#if defined(_NO_EXCEPTION)
#  define _NO_THROW
#  define _THROW_MATRIX_ERROR
#else
#  if defined(_MSC_VER)
#    if _MSC_VER >= 1020
#      include <stdexcept>
#    else
#      include <stdexcpt.h>
#    endif
#  elif defined(__MWERKS__)
#      include <stdexcept>
#  elif (__GNUC__ >= 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 8))
#     include <stdexcept>
#  else
#     include <stdexcep>
#  endif
#  define _NO_THROW               throw ()
#  define _THROW_MATRIX_ERROR     throw (matrix_error)
#endif

#ifndef __MINMAX_DEFINED
#  define max(a, b) (((a) > (b)) ? (a) : (b))
#  define min(a, b) (((a) < (b)) ? (a) : (b))
#endif

#if defined(_MSC_VER)
#undef _MSC_EXTENSIONS
#endif

#if defined(__GNUG__) || defined(__MWERKS__) || (defined(__BORLANDC__) && (__BORLANDC__ >= 0x540))
#define FRIEND_FUN_TEMPLATE <>
#else
#define FRIEND_FUN_TEMPLATE
#endif

#if defined(_MSC_VER) && _MSC_VER <= 1020
#  define _NO_NAMESPACE
#endif

#if !defined(_NO_NAMESPACE)
#if defined(_SGI_BROKEN_STL)
namespace std {}
#endif
using namespace std;
#endif

#ifndef _NO_NAMESPACE
namespace math {
#endif

#if !defined(_NO_EXCEPTION)
class matrix_error : public logic_error
{
    public:
	matrix_error(const string& what_arg) : logic_error(what_arg) {}
};
#define REPORT_ERROR(ErrormMsg)  throw matrix_error(ErrormMsg);
#else
inline void _matrix_error(const char* pErrMsg)
{
    cout << pErrMsg << endl;
    exit(1);
}
#define REPORT_ERROR(ErrormMsg)  _matrix_error( ErrormMsg);
#endif

#if !defined(_NO_TEMPLATE)
#  define MAT_TEMPLATE  template <class T>
#  define matrixT  matrx<T>
#else
#  define MAT_TEMPLATE
#  define matrixT  matrx
#  ifdef MATRIX_TYPE
     typedef MATRIX_TYPE T;
#  else
     typedef double T;
#  endif
#endif


MAT_TEMPLATE
class matrx
{
public:
	matrx(const matrixT& m);
	matrx(size_t row = 6, size_t col = 6);
	~matrx();
	matrixT& operator=(const matrixT& m);
	size_t RowNo() const { return _m->Row; }
	size_t ColNo() const { return _m->Col; }
	T& operator() (size_t row, size_t col);
	T operator() (size_t row, size_t col) const;
	matrixT operator+() _NO_THROW { return *this; }
	matrixT operator-() _NO_THROW;
	matrixT& operator+=(const matrixT& m);
	matrixT& operator-=(const matrixT& m);
	matrixT& operator*=(const matrixT& m);
	matrixT& operator*=(const T& c);
	matrixT& operator/=(const T& c);
	matrixT& operator^=(const size_t& pow);
	matrixT getSymmetric();
	void Null(const size_t& row, const size_t& col);
	void Null();
	void Unit(const size_t& row);
	void Unit();
	matrixT mean();
	matrixT meanSubtracted();
	matrixT scaleImageValues(const T& max);
	void SetSize(size_t row, size_t col);
	matrixT Solve(const matrixT& v) const;
	matrixT Adj();
	matrixT Inv();
	matrixT getRow(int RowNo);
	matrixT* getRows();
	matrixT getColumn(int ColNo);
	matrixT reshape(int width, int height);
	matrixT clamp(const T& min, const T& max);
	T** getMatrix() const;
	void setRow(int RowNo, matrixT &m);
	T Det() const;
	T Norm();
	T Cofact(size_t row, size_t col);
	T Cond();
	bool IsSquare() { return (_m->Row == _m->Col); }
	bool IsSingular();
	bool IsDiagonal();
	bool IsScalar();
	bool IsUnit();
	bool IsNull();
	bool IsSymmetric();
	bool IsSkewSymmetric();
	bool IsUpperTriangular();
	bool IsLowerTriangular();
private:
	struct base_mat
	{
		T** Val;
		size_t Row;
		size_t Col;
		size_t RowSiz;
		size_t ColSiz;
		int Refcnt;
		base_mat(size_t row, size_t col, T** v)
		{
			Row = row;
			RowSiz = row;
			Col = col;
			ColSiz = col;
			Refcnt = 1;
			Val = new T*[row];
			size_t rowlen = col * sizeof(T);
			for (size_t i = 0; i < row; i++)
			{	Val[i] = new T[col];
				if (v)
				{	memcpy(Val[i], v[i], rowlen);
				}
			}
		}
		~base_mat()
		{
			for (size_t i = 0; i < RowSiz; i++)
			{	delete [] Val[i];
			}
			delete [] Val;
		}
	};
	base_mat* _m;
	void clone();
	void realloc(size_t row, size_t col);
	int pivot(size_t row);
};

#if defined(_MSC_VER) && _MSC_VER <= 1020
#  undef  _NO_THROW
#  undef  _THROW_MATRIX_ERROR
#  define _NO_THROW
#  define _THROW_MATRIX_ERROR
#endif

MAT_TEMPLATE inline
matrixT::matrx(size_t row, size_t col)
{
	_m = new base_mat(row, col, 0);
}

MAT_TEMPLATE inline
matrixT::matrx(const matrixT& m)
{
    _m = m._m;
    _m->Refcnt++;
}

/********************* Start of added functions *********************/

MAT_TEMPLATE inline
matrixT matrixT::mean()
/* PURPOSE:		Calculates mean image of all images stored.

   RETURNS:		Mean image matrix of all stored images.

   REMARKS:		First calculates mean pixel value for each
				column.  Then mean pixel divides pixels in
				each column.
*/
{
	matrixT mn(1, _m->Col);
	mn.Null();
	unsigned i;
	unsigned j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
	{		mn._m->Val[0][j] += _m->Val[i][j];
		}
	}
	mn /= _m->Row;
	return mn;
}

MAT_TEMPLATE inline
matrixT matrixT::meanSubtracted()
/* PURPOSE:		Normalizes image intensity values by centering spread
				of pixel intensities of all images about 0.
	
   RETURNS:		CMatrix object with normalized pixel intensity values.
*/
{
	matrixT tmp = *this;
	matrixT mn = mean();
	unsigned i;
	unsigned j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	tmp(i, j) -= mn(0, j);
		}
	}
	return tmp;
}

MAT_TEMPLATE inline
matrixT matrixT::reshape(int width, int height)
/* PURPOSE:		Converts a 1 by M * N matrix into an M by N matrix.

   RECEIVES:	Width and height of matrix to be reshaped into.

   RETURNS:		CMatrix object that is reshaped from a 1 by M * N
				to M * N matrix.

   REMARKS:		Assumes that this matrix is a 1 by width * height
				matrix.
*/
{
	if (_m->Row != 1 || _m->Col != width * height)
	{	REPORT_ERROR("matrixT::reshape : Inconsistent matrix sizes in reshape!");
	}
	matrixT tmp(height, width);
	unsigned i;
	for (i = 0; i < _m->Col; i++)
	{	tmp(i / width, i % width) = _m->Val[0][i];
	}
	return tmp;
}

MAT_TEMPLATE inline
void matrixT::setRow(int RowNo, matrixT &m)
/* PURPOSE:		Set specified row of this matrix object to values
				in matrix "m".

   RECEIVES:	Row number to overwrite with values in matrix "m".

   REMARKS:		CMatrix "m" must be a 1-dimensional with
				same width as this matrix.
*/
{
	if (RowNo < 0 || RowNo >= (int)_m->Row)
	{	REPORT_ERROR("matrixT::setRow : Invalid row index in setRow!");
	}
	if (m.RowNo() != 1 || m.ColNo() != _m->Col)
	{	REPORT_ERROR("matrixT::setRow : Inconsistent matrix sizes in setRow!");
	}
	unsigned i;
	for (i = 0; i < _m->Col; i++)
	{	_m->Val[RowNo][i] = m(0, i);
	}
}

MAT_TEMPLATE inline
matrixT matrixT::getRow(int RowNo)
/* PURPOSE:		To access a copy of a specific matrix row.

   RECEIVES:	Row number to access.

   RETURNS:		CMatrix object representing copy of specified row.
*/
{
	if (RowNo < 0 || RowNo >= (int)_m->Row)
	{	REPORT_ERROR("matrixT::getRow : Invalid row index in getRow!");
	}
	matrixT tmp;
	tmp.SetSize(1, _m->Col);
	unsigned i;
	for (i = 0; i < _m->Col; i++)
	{	tmp(0, i) = _m->Val[RowNo][i];
	}
	return tmp;
}

MAT_TEMPLATE inline
matrixT* matrixT::getRows()
/* PURPOSE:		To access rows of matrix as CMatrix objects.

   RETURNS:		Returns a pointer to an array of CMatrix
				objects representing the rows of the
				original matrix.
*/
{
	matrixT* array = new matrixT[_m->Row];
	unsigned i;
	for (i = 0; i < _m->Row; i++)
	{	array[i] = getRow(i);
	}
	return array;
}

MAT_TEMPLATE inline
matrixT matrixT::getColumn(int ColNo)
/* PURPOSE:		To access a copy of a specific matrix column.

   RECEIVES:	Column number to access.

   RETURNS:		CMatrix object representing copy of specified
				column.
*/
{
	if (ColNo < 0 || ColNo >= (int)_m->Col)
	{	REPORT_ERROR("matrixT::getColumn : Invalid row index in getColumn!");
	}
	matrixT tmp;
	tmp.SetSize(RowNo(), 1);
	unsigned i;
	for (i = 0; i < tmp.RowNo(); i++)
	{	tmp(i, 0) = _m->Val[i][ColNo];
	}
	return tmp;
}

MAT_TEMPLATE inline
matrixT matrixT::getSymmetric()
/* PURPOSE:		To obtain symmetric matrix.

   RETURNS:		Symmetric matrix by multiplying this matrix
				with its transpose.

   REMARKS:		More efficient than doing A * ~A (1 full matrix
				operation instead of 2).
*/
{
	matrixT tmp(_m->Row, _m->Row);
	unsigned i;
	unsigned j;
	unsigned k;
	for(i = 0; i < _m->Row; i++)
	{	for(j = 0; j < _m->Row; j++)
		{	T sum = 0.0;
			for(k = 0; k < _m->Col; k++)
			{	sum += (_m->Val[i][k] * _m->Val[j][k]);
			}
			tmp(i, j) = sum;
		}
	}
	return tmp;
}

MAT_TEMPLATE inline
matrixT matrixT::scaleImageValues(const T& max)
/* PURPOSE:		Some images may be too dark to display.  This function
				brightens an image to its maximum by taking each pixel
				and multiplying it by the ratio of max over the largest
				pixel intensity found in image.

   RETURNS:		CMatrix object with brightened pixel values.
*/
{
	matrixT image = *this;
	T largest = image(0, 0);
	unsigned i;
	unsigned j;
	for (i = 0; i < image.RowNo(); i++)
	{	for (j = 0; j < image.ColNo(); j++)
		{	if (largest < image(i, j))
			{	largest = image(i, j);
			}
		}
	}
	if (largest > max)
	{	return image;
	}
	for (i = 0; i < image.RowNo(); i++)
	{	for (j = 0; j < image.ColNo(); j++)
		{	image(i, j) *= max / largest;
			image(i, j) = abs(image(i, j));
		}
	}
	return image;	
}

MAT_TEMPLATE inline
matrixT matrixT::clamp(const T& min, const T& max)
{
	matrixT temp = *this;
	unsigned i;
	unsigned j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	if (temp._m->Val[i][j] < 0)
			{	temp._m->Val[i][j] = min;
			}
			else if (temp._m->Val[i][j] > max)
			{	temp._m->Val[i][j] = max;
			}
		}
	}
	return temp;
}

MAT_TEMPLATE inline
T** matrixT::getMatrix() const
{
	return _m->Val;
}

/********************* End of added functions *********************/

MAT_TEMPLATE inline
void matrixT::clone()
{
    _m->Refcnt--;
    _m = new base_mat(_m->Row, _m->Col, _m->Val);
}

MAT_TEMPLATE inline
matrixT::~matrx()
{
	if (--_m->Refcnt == 0)
	{	delete _m;
	}
}

MAT_TEMPLATE inline
matrixT& matrixT::operator=(const matrixT& m)
{
	m._m->Refcnt++;
	if (--_m->Refcnt == 0)
	{	delete _m;
	}
	_m = m._m;
	return *this;
}

MAT_TEMPLATE inline
void matrixT::realloc(size_t row, size_t col)
{
	if (row == _m->RowSiz && col == _m->ColSiz)
	{	_m->Row = _m->RowSiz;
		_m->Col = _m->ColSiz;
		return;
	}
	base_mat *m1 = new base_mat(row, col, NULL);
	size_t colSize = min(_m->Col, col) * sizeof(T);
	size_t minRow = min(_m->Row, row);
	size_t i;
	for (i = 0; i < minRow; i++)
	{	memcpy(m1->Val[i], _m->Val[i], colSize);
	}
	if (--_m->Refcnt == 0)
	{	delete _m;
	}
	_m = m1;
	return;
}

MAT_TEMPLATE inline
void matrixT::SetSize(size_t row, size_t col)
{
	size_t i;
	size_t j;
	size_t oldRow = _m->Row;
	size_t oldCol = _m->Col;
	if (row != _m->RowSiz || col != _m->ColSiz)
	{	realloc(row, col);
	}
	for (i = oldRow; i < row; i++)
	{	for (j = 0; j < col; j++)
		{	_m->Val[i][j] = T(0);
		}
	}
	for (i = 0; i < row; i++)
	{	for (j = oldCol; j < col; j++)
		{	_m->Val[i][j] = T(0);
		}
	}
	return;
}

MAT_TEMPLATE inline
T& matrixT::operator()(size_t row, size_t col)
{
	if (row >= _m->Row || col >= _m->Col)
	{	REPORT_ERROR("matrixT::operator(): Index out of range!");
	}
	if (_m->Refcnt > 1)
	{	clone();
	}
	return _m->Val[row][col];
}

MAT_TEMPLATE inline
T matrixT::operator()(size_t row, size_t col) const
{
	if (row >= _m->Row || col >= _m->Col)
	{	REPORT_ERROR("matrixT::operator(): Index out of range!");
	}
	return _m->Val[row][col];
}

MAT_TEMPLATE inline
istream& operator>>(istream& istrm, matrixT& m)
{
	size_t i;
	size_t j;
	for (i = 0; i < m.RowNo(); i++)
	{	for (j = 0; j < m.ColNo(); j++)
		{	T x;
			istrm >> x;
			m(i, j) = x;
		}
	}
	return istrm;
}

MAT_TEMPLATE inline
ostream& operator<<(ostream& ostrm, const matrixT& m)
{
	size_t i;
	size_t j;
	for (i = 0; i < m.RowNo(); i++)
	{	for (j = 0; j < m.ColNo(); j++)
		{	T x = m(i, j);
			ostrm << x << '\t';
		}
		ostrm << endl;
	}
	return ostrm;
}

MAT_TEMPLATE inline
bool operator==(const matrixT& m1, const matrixT& m2)
{
	if (m1.RowNo() != m2.RowNo() || m1.ColNo() != m2.ColNo())
	{	return false;
	}
	size_t i;
	size_t j;
	for (i = 0; i < m1.RowNo(); i++)
	{	for (j = 0; j < m1.ColNo(); j++)
		{	if (m1(i, j) != m2(i, j))
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool operator!=(const matrixT& m1, const matrixT& m2)
{
    return (m1 == m2) ? false : true;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator+=(const matrixT& m)
{
	if (_m->Row != m._m->Row || _m->Col != m._m->Col)
	{	REPORT_ERROR("matrixT::operator+= : Inconsistent matrix sizes in addition!");
	}
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < m._m->Row; i++)
	{	for (j = 0; j < m._m->Col; j++)
		{	_m->Val[i][j] += m._m->Val[i][j];
		}
	}
	return *this;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator-=(const matrixT& m)
{
	if (_m->Row != m._m->Row || _m->Col != m._m->Col)
	{	REPORT_ERROR("matrixT::operator-= : Inconsistent matrix sizes in subtraction!");
	}
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < m._m->Row; i++)
	{	for (j = 0; j < m._m->Col; j++)
		{	_m->Val[i][j] -= m._m->Val[i][j];
		}
	}
	return *this;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator*=(const T& c)
{
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] *= c;
		}
	}
	return *this;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator*=(const matrixT& m)
{
	if (_m->Col != m._m->Row)
	{	REPORT_ERROR("matrixT::operator*= : Inconsistent matrix sizes in multiplication!");
	}
	matrixT temp(_m->Row, m._m->Col);
	size_t i;
	size_t j;
	size_t k;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < m._m->Col; j++)
		{	temp._m->Val[i][j] = T(0);
			for (k = 0; k < _m->Col; k++)
			{	temp._m->Val[i][j] += _m->Val[i][k] * m._m->Val[k][j];
			}
		}
	}
	*this = temp;
	return *this;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator/=(const T& c)
{
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] /= c;
		}
	}
	return *this;
}

MAT_TEMPLATE inline
matrixT& matrixT::operator^=(const size_t& pow)
{
	matrixT temp(*this);
	size_t i;
	for (i = 2; i <= pow; i++)
	{	*this = *this * temp;
	}
	return *this;
}

MAT_TEMPLATE inline
matrixT matrixT::operator-()
{
	matrixT temp(_m->Row, _m->Col);
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	temp._m->Val[i][j] = -_m->Val[i][j];
		}
	}
	return temp;
}

MAT_TEMPLATE inline
matrixT operator+(const matrixT& m1, const matrixT& m2)
{
	matrixT temp = m1;
	temp += m2;
	return temp;
}

MAT_TEMPLATE inline
matrixT operator-(const matrixT& m1, const matrixT& m2)
{
	matrixT temp = m1;
	temp -= m2;
	return temp;
}

MAT_TEMPLATE inline
matrixT operator*(const matrixT& m, const T& no)
{
	matrixT temp = m;
	temp *= no;
	return temp;
}

MAT_TEMPLATE inline
matrixT operator*(const T& no, const matrixT& m)
{
	return (m * no);
}

MAT_TEMPLATE inline
matrixT operator*(const matrixT& m1, const matrixT& m2)
{
	matrixT temp = m1;
	temp *= m2;
	return temp;
}

MAT_TEMPLATE inline
matrixT operator/(const matrixT& m, const T& no)
{
	return (m * (T(1) / no));
}

MAT_TEMPLATE inline
matrixT operator/(const T& no, const matrixT& m)
{
	return (!m * no);
}

MAT_TEMPLATE inline
matrixT operator/(const matrixT& m1, const matrixT& m2)
{
	return (m1 * !m2);
}

MAT_TEMPLATE inline
matrixT operator^(const matrixT& m, const size_t& pow)
{
	matrixT temp = m;
	temp ^= pow;
	return temp;
}

MAT_TEMPLATE inline
matrixT operator~(const matrixT& m)
{
	matrixT temp(m.ColNo(), m.RowNo());
	size_t i;
	size_t j;
	for (i = 0; i < m.RowNo(); i++)
	{	for (j = 0; j < m.ColNo(); j++)
		{	T x = m(i, j);
			temp(j, i) = x;
		}
	}
	return temp;
}

MAT_TEMPLATE inline
matrixT operator!(const matrixT m)
{
	matrixT temp = m;
	return temp.Inv();
}

MAT_TEMPLATE inline
T dot(const matrixT& m1, const matrixT& m2)
{
	T tmp = 0.0;
	size_t i;
	for (i = 0; i < m1.ColNo(); i++)
	{	tmp += (m1(0, i) * m2(i, 0));
	}
	return tmp;
}

MAT_TEMPLATE inline
matrixT matrixT::Inv()
{
	size_t i;
	size_t j;
	size_t k;
	T a1;
	T a2;
	T* rowptr;
	if (_m->Row != _m->Col)
	{	REPORT_ERROR("matrixT::operator!: Inversion of a non-square matrix");
	}
	matrixT temp(_m->Row, _m->Col);
	if (_m->Refcnt > 1)
	{	clone();
	}
	temp.Unit();
	for (k = 0; k < _m->Row; k++)
	{	int indx = pivot(k);
		if (indx == -1)
		{	REPORT_ERROR("matrixT::operator!: Inversion of a singular matrix");
		}
		if (indx != 0)
		{	rowptr = temp._m->Val[k];
			temp._m->Val[k] = temp._m->Val[indx];
			temp._m->Val[indx] = rowptr;
		}
		a1 = _m->Val[k][k];
		for (j = 0; j < _m->Row; j++)
		{	_m->Val[k][j] /= a1;
			temp._m->Val[k][j] /= a1;
		}
		for (i = 0; i < _m->Row; i++)
		{	if (i != k)
			{	a2 = _m->Val[i][k];
				for (j = 0; j < _m->Row; j++)
				{	_m->Val[i][j] -= a2 * _m->Val[k][j];
					temp._m->Val[i][j] -= a2 * temp._m->Val[k][j];
				}
			}
		}
	}
	return temp;
}

MAT_TEMPLATE inline
matrixT matrixT::Solve(const matrixT& v) const
{
	size_t i;
	size_t j;
	size_t k;
	T a1;
	if (!(_m->Row == _m->Col && _m->Col == v._m->Row))
	{	REPORT_ERROR("matrixT::Solve():Inconsistent matrices!");
	}
	matrixT temp(_m->Row, _m->Col+v._m->Col);
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	temp._m->Val[i][j] = _m->Val[i][j];
		}
		for (k = 0; k < v._m->Col; k++)
		{	temp._m->Val[i][_m->Col + k] = v._m->Val[i][k];
		}
	}
	for (k = 0; k < _m->Row; k++)
	{	int indx = temp.pivot(k);
		if (indx == -1)
		{	REPORT_ERROR("matrixT::Solve(): Singular matrix!");
		}
		a1 = temp._m->Val[k][k];
		for (j = k; j < temp._m->Col; j++)
		{	temp._m->Val[k][j] /= a1;
		}
		for (i = k + 1; i < _m->Row; i++)
		{	a1 = temp._m->Val[i][k];
			for (j = k; j < temp._m->Col; j++)
			{	temp._m->Val[i][j] -= a1 * temp._m->Val[k][j];
			}
		}
	}
	matrixT s(v._m->Row, v._m->Col);
	for (k = 0; k < v._m->Col; k++)
	{	for (int m = int(_m->Row) - 1; m >= 0; m--)
		{	s._m->Val[m][k] = temp._m->Val[m][_m->Col + k];
			for (j = m + 1; j < _m->Col; j++)
			{	s._m->Val[m][k] -= temp._m->Val[m][j] * s._m->Val[j][k];
			}
		}
	}
	return s;
}

MAT_TEMPLATE inline
void matrixT::Null(const size_t& row, const size_t& col)
{
	if (row != _m->Row || col != _m->Col)
	{	realloc(row, col);
	}
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] = T(0);
		}
	}
	return;
}

MAT_TEMPLATE inline
void matrixT::Null()
{
	if (_m->Refcnt > 1)
	{	clone();
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] = T(0);
		}
	}
	return;
}

MAT_TEMPLATE inline void
matrixT::Unit(const size_t& row) _NO_THROW
{
    if (row != _m->Row || row != _m->Col)
	{	realloc(row, row);
	}
    if (_m->Refcnt > 1) 
	{	clone();
	}
	size_t i;
	size_t j;
    for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] = i == j ? T(1) : T(0);
		}
	}
    return;
}

MAT_TEMPLATE inline void
matrixT::Unit() _NO_THROW
{
    if (_m->Refcnt > 1)
	{	clone();  
	}
    size_t row = min(_m->Row, _m->Col);
    _m->Row = _m->Col = row;
	size_t i;
	size_t j;
    for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	_m->Val[i][j] = i == j ? T(1) : T(0);
		}
	}
    return;
}

MAT_TEMPLATE inline
int matrixT::pivot(size_t row)
{
	double temp;
	double amax = -1;
	int k = int(row);
	size_t i;
	for (i = row; i < _m->Row; i++)
	{	if ((temp = abs( _m->Val[i][row])) > amax && temp != 0.0)
		{	amax = temp;
			k = i;
		}
	}
	if (_m->Val[k][row] == T(0))
	{	return -1;
	}
	if (k != int(row))
	{	T* rowptr = _m->Val[k];
		_m->Val[k] = _m->Val[row];
		_m->Val[row] = rowptr;
		return k;
	}
	return 0;
}

MAT_TEMPLATE inline
T matrixT::Det() const
{
	size_t i;
	size_t j;
	size_t k;
	T piv;
	T detVal = T(1);
	if (_m->Row != _m->Col)
	{	REPORT_ERROR("matrixT::Det(): Determinant a non-square matrix!");
	}
	matrixT temp(*this);
	if (temp._m->Refcnt > 1)
	{	temp.clone();
	}
	for (k = 0; k < _m->Row; k++)
	{	int indx = temp.pivot(k);
		if (indx == -1)
		{	return 0;
		}
		if (indx != 0)
		{	detVal = - detVal;
		}
		detVal = detVal * temp._m->Val[k][k];
		for (i = k + 1; i < _m->Row; i++)
		{	piv = temp._m->Val[i][k] / temp._m->Val[k][k];
			for (j = k + 1; j < _m->Row; j++)
			{	temp._m->Val[i][j] -= piv * temp._m->Val[k][j];
			}
		}
	}
	return detVal;
}

MAT_TEMPLATE inline
T matrixT::Norm()
{
	T retVal = T(0);
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	retVal += _m->Val[i][j] * _m->Val[i][j];
		}
	}
	retVal = sqrt(retVal);
	return retVal;
}

MAT_TEMPLATE inline
T matrixT::Cond()
{
	matrixT inv = ! (*this);
	return (Norm() * inv.Norm());
}

MAT_TEMPLATE inline
T matrixT::Cofact(size_t row, size_t col)
{
	size_t i;
	size_t j;
	size_t i1;
	size_t j1;
	if (_m->Row != _m->Col)
	{	REPORT_ERROR("matrixT::Cofact(): Cofactor of a non-square matrix!");
	}
	if (row > _m->Row || col > _m->Col)
	{	REPORT_ERROR("matrixT::Cofact(): Index out of range!");
	}
	matrixT temp(_m->Row - 1, _m->Col - 1);
	for (i = i1 = 0; i < _m->Row; i++)
	{	if (i == row)
		{	continue;
		}
		for (j = j1 = 0; j < _m->Col; j++)
		{	if (j == col)
			{	continue;
			}
			temp._m->Val[i1][j1] = _m->Val[i][j];
			j1++;
		}
		i1++;
	}
	T cof = temp.Det();
	if ((row + col) % 2 == 1)
	{	cof = -cof;
	}
	return cof;
}

MAT_TEMPLATE inline
matrixT matrixT::Adj()
{
	if (_m->Row != _m->Col)
	{	REPORT_ERROR("matrixT::Adj(): Adjoin of a non-square matrix.");
	}
	matrixT temp(_m->Row, _m->Col);
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	temp._m->Val[j][i] = Cofact(i, j);
		}
	}
	return temp;
}

MAT_TEMPLATE inline
bool matrixT::IsSingular()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	return (Det() == T(0));
}

MAT_TEMPLATE inline
bool matrixT::IsDiagonal()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	if (i != j && _m->Val[i][j] != T(0))
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsScalar()
{
	if (!IsDiagonal())
	{	return false;
	}
	T v = _m->Val[0][0];
	size_t i;
	for (i = 1; i < _m->Row; i++)
	{	if (_m->Val[i][i] != v)
		{	return false;
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsUnit()
{
	if (IsScalar() && _m->Val[0][0] == T(1))
	{	return true;
	}
	return false;
}

MAT_TEMPLATE inline
bool matrixT::IsNull()
{
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	if (_m->Val[i][j] != T(0))
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsSymmetric()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	if (_m->Val[i][j] != _m->Val[j][i])
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsSkewSymmetric()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	size_t i;
	size_t j;
	for (i = 0; i < _m->Row; i++)
	{	for (j = 0; j < _m->Col; j++)
		{	if (_m->Val[i][j] != -_m->Val[j][i])
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsUpperTriangular()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	size_t i;
	size_t j;
	for (i = 1; i < _m->Row; i++)
	{	for (j = 0; j < i - 1; j++)
		{	if (_m->Val[i][j] != T(0))
			{	return false;
			}
		}
	}
	return true;
}

MAT_TEMPLATE inline
bool matrixT::IsLowerTriangular()
{
	if (_m->Row != _m->Col)
	{	return false;
	}
	size_t i;
	size_t j;
	for (j = 1; j < _m->Col; j++)
	{	for (i = 0; i < j - 1; i++)
		{	if (_m->Val[i][j] != T(0))
			{	return false;
			}
		}
	}
	return true;
}

#ifndef _NO_NAMESPACE
}
#endif

using namespace math;

typedef matrx<double> CMatrix;

#endif
