/********************************************************************************

PROJECT:		CS 298 Project

FILE:			reconstruct.cpp

PURPOSE:		Reconstructs and ages an input image.

COMPILER:		Visual C++ .NET 2003

PROGRAMMER:		Wallun Chan

START DATE:		9/1/05	

********************************************************************************/

#include "stdafx.h"
#include "PCA_ReconstructorDoc.h"
#include "reconstruct.h"
#include "featurelocate.h"
#include "constants.h"

CReconstruct::CReconstruct(CFaceMap* fp) : faceMap(fp)
{
}

CReconstruct::~CReconstruct()
{
}

BOOL CReconstruct::init(const CString& file, 
CImageMatrix& input, CEigenParams& eigenParams)
{
	if (faceMap == 0)
		return FALSE;
	if (!input.loadImage(file))
		return FALSE;
	CEigenUtil eigenUtil(faceMap);
	eigenUtil.setEigenParams(eigenParams);
	return TRUE;
}

BOOL CReconstruct::reconstruct(const CString& file, 
CMatrix* iSpc, CMatrix* eSpc, CImageMatrix& image, 
CImageMatrix& features, CFaceParams& fParams, 
int noExtraction, double sigma, int* threshold, 
int* stopFlag, FuncPtr fp, int start, int end)
{
	CImageMatrix input;
	CEigenParams eigenParams;
	if (!init(file, input, eigenParams))
		return FALSE;
	CData data;
	CFeatureLocate faceUtil;
	faceUtil.setImage(&input);
	if (!faceUtil.setData(&data, sigma, threshold))
		return FALSE;
	fParams = data.param;
	return reconstruct(eigenParams, input, data.param.rect, input.getWidth(), 
		input.getHeight(), data.param.angle, iSpc, eSpc, image, features, 
		noExtraction, sigma, threshold, stopFlag, fp, start, end);
}

BOOL CReconstruct::reconstruct(const CString& file, CRect rect[], 
double angle, CMatrix* iSpc, CMatrix* eSpc, CImageMatrix& image,
CImageMatrix& features, int noExtraction, double sigma, 
int* threshold, int* stopFlag, FuncPtr fp, int start, int end)
{
	CImageMatrix input;
	CEigenParams eigenParams;
	if (!init(file, input, eigenParams))
		return FALSE;
	return reconstruct(eigenParams, input, rect, input.getWidth(), 
		input.getHeight(), angle, iSpc, eSpc, image, features, 
		noExtraction, sigma, threshold, stopFlag, fp, start, end);
}

BOOL CReconstruct::reconstruct(const CEigenParams& eigenParams, 
CImageMatrix& input, CRect* rect, int width, int height, 
double angle, CMatrix* iSpc, CMatrix* eSpc, CImageMatrix& image,
CImageMatrix& features, int noExtraction, double sigma, 
int* threshold, int* stopFlag, FuncPtr fp, int start, int end)
{
	image.createRGBImage(width, height);
	features.createRGBImage(width, height);
	CMatrix trainIm, coef;
	CImageMatrix rotated, region, scaled;
	CImageMatrix left, right, rec, isolated;
	input.rotateImage(rotated, angle);
	double step = (double)(end - start) / FEATURE_NUM;
	double count = start;
	int i, n;
	if (noExtraction)
		n = FEATURE_NUM - 1;
	else
		n = 0;
	for (i = FEATURE_NUM - 1; i >= n; i--)
	{	region.setImage(rotated, rect[i]);
		double xs = (double)(eigenParams.width[i] + 1) / 
			(rect[i].Width() + 1);
		double ys = (double)(eigenParams.height[i] + 1) / 
			(rect[i].Height() + 1);
		region.scaleImage(scaled, xs, ys);
		CImageMatrix::demultiplex(scaled, scaled, trainIm);
		coef.SetSize(1, eSpc[i].RowNo());
		CMatrix mn = iSpc[i].mean();
		CMatrix gamma = ~(trainIm - mn);
		for (int j = 0; j < (int)eSpc[i].RowNo(); j++)
			coef(0, j) = dot(eSpc[i].getRow(j), gamma);
		CMatrix recon = (coef * eSpc[i]) + mn;
		CImageMatrix::multiplex(recon.reshape(2 * (eigenParams.width[i] + 1), 
			(eigenParams.height[i] + 1) * COLOR_CHANNEL_NUM).clamp(
			0, PIXEL_MAX), left, right);
		right.scaleImage(rec, 1.0 / xs, 1.0 / ys);
		if (i == FEATURE_NUM - 1)
			image.overlay(rect[i].TopLeft(), rec);
		else
		{	rec.isolateFeature(isolated, sigma, threshold);	
			image.overlay(rect[i].TopLeft(), isolated, TRUE);	
			features.overlay(rect[i].TopLeft(), isolated, TRUE);
		}
		count += step;
		fp((int)count, 0);
	}
	return TRUE;
}